c  Based on EPRBL - Version 1.4 4/16/93
c----------------------------------------------------------------------
c
c                            ===============
c                            PROGRAM : EPRBS
c                            ===============
c
c       This program does the EPR "field sweep" conjugate gradients
c       calculation in order to determine the maximum projection of each 
c       basis vector on the solution vector across the EPR spectrum,
c       for EPRSR package.
c       The program stores this information in a basis set named
c       <filename>.bss. Each projection is normalized to the largest
c       projection found in the basis set.
c
c       The basis set projections are then used to determine the minimum 
c       truncation set (MTS) indices as originally done by the program
c       TNLL in Versions 1.0 through 1.3. This produces an MTS database
c       file ( <filename>.mts ) containing the MTS parameters as a function
c       of "significance factor", i.e., the minimum projection for which
c       a basis element is considered to have a "significant" overlap with
c       the solution vector. Sixteen different MTS sets are specified for
c       significance factors of 2**-(i+1) for i from 1 to 16. 
c
c       The basis set projections are also used to determine a "pruned"
c       basis set that is obtained by removing all basis vectors whose
c       maximum projection on the solution vector across the spectrum is
c       less than a specified cutoff value, btol. This procedure may often 
c       produce basis sets that are considerably smaller than those obtained
c       using the simple MTS truncation rules. If btol is zero, then no
c       pruning will take place. 
c
c       Nothing new has been added for EPRSR calculations, except
c       for the increased number of indexes:
c
c       minl  & maxl    : min and max value for L
c       minm  & maxm    : min and max value for M
c       mink  & maxk    : min and max value for K
c       minj  & maxj    : min and max value for J
c       minn  & maxn    : min and max value for N
c       minpi & maxpi   : min and max value for pI
c
c       Written by SHL based on original EPRBL, TNLL programs from DJS
c       (through version 1.3). Modified by DEB 16-APR-93 for EPRLL calculation.
c
c       Modified by AP 17-FEB-94 for EPRSR calculation.
c
c
c       Includes :
c               stddim.inc
c               stdio.inc
c               eprmat.inc
c               eprdat.inc
c               fnames.inc
c               indexl.inc
c               pidef.inc
c               timer.inc (UNIX-specific timing with dtime function)
c
c       Uses :
c               getids.f
c               setnam.f
c               rddat.f
c               wrdat.f
c               lbasix.f
c               matrsr.f
c               stvect.f
c               cspccg.f
c               zdotu.f
c               zaxpy.f
c               scmvm.f
c               eprfss.f
c               mtss.f
c
c----------------------------------------------------------------------
c
      program eprbl
c
      implicit none
c
      include 'stddim.inc'
      include 'stdio.inc'
      include 'eprdat.inc'
      include 'eprmat.inc'
      include 'pidef.inc'
      include 'fnames.inc'
      include 'indexl.inc'
      include 'timer.inc'  
      include 'rndoff.inc'  
c
      integer mxcut
      parameter (mxcut=16)
c
      double precision zero,one
      parameter (zero=0.0D0,one=1.0D0)
c
      logical fexist,fmomd,report
c
      integer i,icut,ierr,iort
      integer maxle,minle,maxlo,minlo,maxm,minm,maxk,mink,maxj,minj,
     #        maxn,minn,maxpi,minpi,maxi,mini
      integer ncalc,nfiles,nbas,nmts,nprune,nstpmx,ntotal
      integer keep(mxdim)
c
      real cpufs
      double precision spect(2,mxpt),basis(2,mxdim),b(2,mxdim),
     #     bcut,bmax,cspsi,dfield,terrmx
      character flgstr*5
c
      integer jk2mn,npts,jk2,jk2sp
      double precision y(2,mxdim),vnorm
c
      integer ipar
      complex*16 temp,zdotu
      external zdotu,ipar
c
c#####################################################################
c
c----------------------------------------------------------------------
c Timing call changed for compatibility with UNIX
c      call cputime(cpu0,ircode)
c----------------------------------------------------------------------
      cputot = dtime( tarray )
c
      write (luttyo,1000)
      write (luttyo,1010)
      write (luttyo,1000)
c
      call getids(nfiles,mxcalc)
c
c======================================================================
c   Loop over calculations
c======================================================================
c
      do 100 ncalc=1,nfiles
c
c---------------------------------------------------------------------
c     construct file names
c---------------------------------------------------------------------
c
         fileid = files(ncalc)
c
         call setnam
         call setflg( flags(ncalc) )
c
c----------------------------------------------------------------------
c     open log file and write message
c----------------------------------------------------------------------
c
         open (unit=lulog,file=rlname(:namlth),status='unknown',
     #      access='sequential',form='formatted')
         rewind (unit=lulog)
c
         write (lulog,1000)
         write (luttyo,1030) prname(:namlth)
         write (lulog,1030) prname(:namlth)
         write (lulog,1000)
c
c----------------------------------------------------------------------
c     read parameters from file
c----------------------------------------------------------------------
c
         inquire(file=prname(:namlth),exist=fexist)
c
         if (fexist) then
            call rddat(prname,namlth,ierr)
         else
            write (luttyo,1040) prname(:namlth)
            write (lulog,1040) prname(:namlth)
            go to 999
         endif
c
c----------------------------------------------------------------------
c     check if a field-swept calculation is specified
c----------------------------------------------------------------------
 c
      if (itype.ne.2) then
         write (luttyo,1020) prname(:namlth)
         write (lulog,1020) prname(:namlth)
         go to 999
      end if
c
c----------------------------------------------------------------------
c  Set up calculation by determining initial basis indices, 
c  building the stochastic Liouville matrix, and the starting vector
c----------------------------------------------------------------------
c
        call lbasix(ixname,namlth,0)
c
        if (ndim.gt.mxdim) then
          write (luttyo,1060) ndim,mxdim
          write (lulog,1060) ndim,mxdim
          go to 999
        end if
c
c---------------------------------------------------------------------
c     calculate d20k
c---------------------------------------------------------------------
c
        npts=512
c
        if (kk0.eq.0) then
           jk2mn=1
           jk2sp=1
        else
           jk2mn=-1
           jk2sp=-2
        endif
c
c       do 888 jk2 = 1,jk2mn,jk2sp
        do 888 jk2 = 1,1
c
        call matr2k(ierr,jk2)
c
        if (ierr.eq.0) then
          write (luttyo,1050) ndim,neltot,cpu1
          write (lulog,1050) ndim,neltot,cpu1
        else
          write (luttyo,1070) cpu1
          write (lulog,1070) cpu1
          if (ierr.eq.1) then
            write (luttyo,1080) mxel
            write (lulog,1080) mxel
          else if (ierr.eq.2) then
            write (luttyo,1090) mxdim
            write (lulog,1090) mxdim
          end if
          go to 999
        end if
c
c----------------------------------------------------------------------
c  Building the starting vector
c----------------------------------------------------------------------
c
        do 40 i=1,mxdim
          y(1,i)=zero
          y(2,i)=zero
 40     continue
c
        call stvect(b)
c
        call scmvmd(b,y,ndim,jk2)
c
        nelv=0
c
        vnorm=zero
        do 45 i=1,ndim
          vnorm=vnorm+y(1,i)*y(1,i)
          b(1,i)=y(1,i)
          b(2,i)=y(2,i)
          y(1,i)=zero
          y(2,i)=zero
          if (abs(b(1,i)).gt.rndoff) nelv=nelv+1
 45     continue
c
        vnorm=one/dsqrt(vnorm)
        write(*,*) 'vnorm=',vnorm
        do i=1,ndim
         b(1,i)=b(1,i)*vnorm
        enddo
c
        do i=ndim+1,mxdim
          b(1,i)=zero
          b(2,i)=zero
          y(1,i)=zero
          y(2,i)=zero
        enddo
c
       cpu2=dtime( tarray )
       write (luttyo,2010) nelv,cpu2
       write (lulog,2010) nelv,cpu2
c
c----------------------------------------------------------------------
c  Building the stochastic Liouville matrix
c----------------------------------------------------------------------
c
        call matrsr(ierr,jk2)
c
        cpu1 = dtime( tarray )
c
        if (ierr.eq.0) then
             write (luttyo,1050) ndim,neltot,cpu1
             write (lulog,1050) ndim,neltot,cpu1
        else 
          write (luttyo,1070) cpu1
          write (lulog,1070) cpu1
          if (ierr.eq.1) then
             write (luttyo,1080) mxel
             write (lulog,1080) mxel
          else if (ierr.eq.2) then
             write (luttyo,1090) mxdim
             write (lulog,1090) mxdim
          end if
          go to 999
       end if
c
c----------------------------------------------------------------------
c     Zero out spectrum array
c----------------------------------------------------------------------
c
        do 120 i=1,mxpt
          spect(1,i)=zero
          spect(2,i)=zero
 120    continue
c
c----------------------------------------------------------------------
c   Determine basis projections with a swept-field calculation
c----------------------------------------------------------------------
c
       bmax=zero
       call eprfss( b,basis,spect,bmax,terrmx,nstpmx,cpufs,ntotal, 
     #              report )
       cputot=cputot+cpufs
c
 888   continue
c
c----------------------------------------------------------------------
c   Prune the basis set by marking the elements that have a projection
c   greater than btol
c----------------------------------------------------------------------
c
      do 4 i=1,ndim
         keep(i)=0
         write(*,*) i,basis(1,i),basis(2,i) 
 4    continue
c
      nprune=0
      do 5 i=1,ndim 
         if (basis(2,i).ge.btol*bmax) then
            keep(i)=1
            nprune=nprune+1
         end if
 5    continue
c
c======================================================================
c   Determine the MTS as a function of 
c   significance factor and write the MTS database file ( <name>.mts )
c======================================================================
c
         write (luttyo,5000) tsname(:namlth)
         open (unit=ludisk,file=tsname(:namlth),status='unknown',
     #        access='sequential',form='formatted')
          write(ludisk,4200)
c
          do 10 i=1,mxcut
             bcut = 0.5d0**(i-1)
             nmts=-1
             call mtss( basis,ndim,bcut*bmax,maxle,minle,maxlo,minlo,
     #            maxm,minm,maxk,mink,maxj,minj,maxn,minn,maxi,mini,
     #            maxpi,minpi,nbas,nmts )
c
             write(ludisk,4210) bcut,minle,maxle,minlo,maxlo,
     #                        minm,maxm,mink,maxk,minj,maxj,minn,maxn,
     #                        mini,maxi,minpi,maxpi,nmts,nbas
 10       continue
          write (ludisk,4220)
          close (ludisk)
c
c     *** Write out spectrum
c
       dfield=(fieldf-fieldi)/dble(nfield-1)
       write (luttyo,5000) spname(:namlth)
       open (unit=ludisk,file=spname(:namlth),status='unknown',
     #      access='sequential',form='formatted')
       write (ludisk,3200) (b0+fieldi+(i-1)*dfield,spect(1,i),
     #      spect(2,i),i=1,nfield)
       close (unit=ludisk)
c
c----------------------------------------------------------------------
c     Condense basis projection and index arrays to retain only
c     the pruned basis 
c----------------------------------------------------------------------
       nprune=0
       do 25 i=1,ndim
          if (keep(i).ne.0) then
             nprune=nprune+1
             basis(1,nprune)=basis(1,i)
             basis(2,nprune)=basis(2,i)
             l1(nprune)=l1(i)
             m1(nprune)=m1(i)
             k1(nprune)=k1(i)
             j1(nprune)=j1(i)
             n1(nprune)=n1(i)
             i1(nprune)=i1(i)
             pi1(nprune)=pi1(i)
             im1(nprune)=im1(i)
             qi1(nprune)=qi1(i)
             ps1(nprune)=ps1(i)
             qs1(nprune)=qs1(i)
          end if
 25    continue
c
c      *** Write out basis set projections
c
       write (luttyo,5000) bsname(:namlth)
       open (unit=ludisk,file=bsname(:namlth),status='unknown',
     #      access='sequential',form='unformatted')
       write (ludisk) nprune,lemx,lomx,mmx,kmx,jmx,nmx,imx,ipnmx
       write (ludisk) (i,l1(i),m1(i),k1(i),j1(i),n1(i),i1(i),
     #                      basis(1,i),basis(2,i),i=1,nprune)
       close (unit=ludisk)
 4001     format(8(i6),i6)
 4002     format(7(i6),g14.7,g14.7)
c
       write (luttyo,4000) ndim,nprune
       write (lulog,4000) ndim,nprune
c
c----------------------------------------------------------------------
c   Find MTS corresponding to pruned basis set
c----------------------------------------------------------------------
c
c      ndim=nprune
       nmts=-1
       call mtss( basis,ndim,zero,maxle,minle,maxlo,minlo,maxm,minm,
     #      maxk,mink,maxj,minj,maxn,minn,maxi,mini,maxpi,minpi,
     #      nbas,nmts )
c
       write(luttyo,4200)
       write(luttyo,4210) btol,minle,maxle,minlo,maxlo,minm,maxm,
     #                    mink,maxk, minj,maxj,minn,maxn,
     #                    mini,maxi,minpi,maxpi,nmts,nbas
       write(luttyo,4220)
c
        write(lulog,4200)
        write(lulog,4210) btol,minle,maxle,minlo,maxlo,minm,maxm,
     #                    mink,maxk, minj,maxj,minn,maxn,
     #                    mini,maxi,minpi,maxpi,nmts,nbas
        write(lulog,4220)
c
c---------------------------------------------------------------------
c     Output updated parameter file and basis index file
c---------------------------------------------------------------------
c
c...... no update of truncation values ..........
c
c       lemx=maxle
c       lomx=maxlo
c       mmx=maxm
c       kmx=maxk
c       jmx=maxj
c       nmx=maxn
c       imx=maxi
c       ipnmx=maxpi
c
c................................................
c
      write (luttyo,5000) prname(:namlth)
      call wrdat(prname,namlth)
c     
c     ***  Write out basis set indices
c
      write (luttyo,5000) ixname(:namlth)
      open (unit=ludisk,file=ixname(:namlth),status='unknown',
     #     access='sequential',form='unformatted')
      write (ludisk) ndim,lemx,lomx,mmx,kmx,jmx,nmx,imx,ipnmx
      write (ludisk) (l1(i),m1(i),k1(i),j1(i),n1(i),i1(i),
     #       pi1(i),im1(i),qi1(i),ps1(i),qs1(i),i=1,ndim)
      close (unit=ludisk)
 5001     format(8(i4),i4)
 5002     format(10(i4),i4)
c
      write (luttyo,5100) cputot
      write (lulog,5100) cputot
c
      write(luttyo,1000)
      write(lulog,1000)	
c
 100  continue
c
 999  close (unit=lulog)
c
      stop
c
c=====================================================================
c
 1000 format(/,2x,70('#'),/)
 1010 format(25x,'program : EPRBS',/,25x,'---------------')
c
 1020 format(//,10x,
     #     '*** ''',a,''' is not set for EPRBS (itype=2) ***')
 1030 format(25x,'file : ',a)
 1040 format(/,10x,'*** file ',a,' not found ***',/)
 1050 format(/,2x,'Dimension of the matrix : ',i6,
     #       /,2x,'Total number of non-zero entries : ',i8,
     #       /,2x,'CPU time (seconds) : ',f10.2)
 1060 format(/,20x,'*** Basis set is too large ***'/
     #       20x, 'Specified dimension=',i7,'; maximum =',i7/)
 1070 format(//,10x,
     #     '*** Serious error encountered in matrix generation ***'
     #     //20x,'CPU time (seconds) : ',f10.2,/)
 1080 format(/,10x,
     #  '*** Number of off-diagonal matrix elements exceeds ',i8,' ***')
 1090 format(/,20x,'*** Matrix dimension exceeds ',i6,' ***')
c
 1200 format(/,2x,'Dimension of the matrix : ',i5,
     #       /,2x,'Total number of non-zero entries : ',i6,
     #       /,2x,'Number of non-zero elements in starting vector ',
     #       ': ',i5,/,2x,'CPU time (seconds) : ',f10.2)
c
 2000 format(/,2x,70('-'),/,4x,'i',3x,'angle',4x,'neltot',3x,
     #      'nelv',3x,'updim',3x,'pdim',3x,'totdim',4x,'cpu',
     #      /,2x,70('-'),/)  
 2010 format(/,2x,'Number of non-zero elements in ',
     #     'starting vector : ',i5,
     #     /,2x,'CPU time for starting vector (seconds) : ',f10.2)
c2010 format(2x,i3,3x,f5.2,3x,i7,3x,i3,4x,i4,3x,i4,5x,i4,3x,f6.2)
c
 3200 format(1x,3(g14.7,','))
c
 4000 format(/,2x,'unpruned dimension = ',i6,
     #           5x,'pruned dimension = ',i5)
 4200 format(/,2x,76('-')/
     #       3x,'error',7x,'LE',6x,'LO',6x,'M',7x,'K',7x,
     #                     'J',7x,'N',7x,'I',7x,'pI',
     #       4x,'MTS',2x,'pruned'/,12x,8('min max '),' dim',3x,'dim'/)
 4210 format(g10.3,16(i4),2i6)
 4220 format(2x,76('-'))
 5000 format(/,13x,'*** writing file ',a,' ***'/)
 5100 format(2x,'Total CPU time for EPRBS calculation (seconds) : ',
     #     f10.2)
c
c=====================================================================
c
      end










