c Based on EPRCGL - Version 1.4  4/13/93
c*********************************************************************
c
c                          ================
c                          PROGRAM : EPRCGS
c                          ================
c                    *** Pruned basis version ***
c
c       This program does the EPR spectral calculation using the
c       complex symmetric conjugate gradients algorithm and the 
c       reconstruction of the "Lanczos" tridiagonal matrix, in
c       the presence of a rotating SRLS.
c
c       written  by DJS 24-OCT-87
c       modified by DEB 13-MAR-92 to allow longer filenames
c                       22-OCT-92 Handles case of version incompatibility
c       modified by AP  17-FEB-94 for EPRSR calculation
c
c       Includes :
c               stddim.inc
c               stdio.inc
c               eprmat.inc
c               eprdat.inc
c               fnames.inc
c               timer.inc (UNIX-specific timing with dtime function)
c
c       Uses :
c               getids.f
c               setnam.f
c               rddat.f
c               wrdat.f
c               matrll.f
c               stvect.f
c               cscg.f
c
c*********************************************************************
c
      subroutine eprcgs (kk0)
c
      implicit none
c
      include 'stddim.inc'
      include 'stdio.inc'
      include 'eprdat.inc'
      include 'eprmat.inc'
      include 'fnames.inc'
      include 'timer.inc'
      include 'rndoff.inc'
      include 'pidef.inc'
c
      integer kk0
      integer jk2,jk2mn,jk2sp
      double precision zero,vnorm
      parameter (zero=0.0D0)
c
      logical fexist
c
      double precision x(2,mxdim),y(2,mxdim),alpha(2,mxstep),
     #                 beta(2,mxstep)
c
      integer i,j,nfiles,ncalc,ierr,ndone
      double precision error
      character*1 chr1,chr2
c
      integer npts,nstepl,ideriv,ndiag,isort
c
      double precision db,bsweep,tr,ti,two
      parameter (two=2.0D0)
      double precision field(mxpt),spect(2,mxpt),
     #       field0(mxpt),spect0(2,mxpt),
     #       temp1(2,mxstep),temp2(2,mxstep),w(2,mxstep)
c
      double precision taum,tauc,tau,orpr,sd1,sd2,sd3,one,c(5)
      parameter(one=1.0D0)

      double precision order
      external order
c
c#####################################################################
c
c........................................
c Timing call specific to UNIX systems
c     cputot = dtime( tarray )
c........................................
c
      write (luttyo,1000)
      write (luttyo,1010)
c
      call getids(nfiles,mxcalc)
      if (nfiles.eq.0) goto 9999
c
c======================================================================
c     loop over calculations
c======================================================================
c
      do 100 ncalc=1,nfiles
c
        fileid = files(ncalc)
c
c---------------------------------------------------------------------
c     construct file names
c---------------------------------------------------------------------
c
        call setnam
        call setflg( flags(ncalc) )
c
c----------------------------------------------------------------------
c     open log file and write out header  
c----------------------------------------------------------------------
c
	open (unit=lulog,file=rlname(:namlth),access='sequential',
     #        status='unknown',form='formatted')
	rewind (unit=lulog)
c
        write (luttyo,1000)
	write (lulog,1000)
        write (luttyo,5000) ncalc,prname(:namlth)
    	write (lulog,5000) ncalc,prname(:namlth)
c
c----------------------------------------------------------------------
c     read in parameters from matrix
c----------------------------------------------------------------------
c
	inquire(file=prname(:namlth),exist=fexist)
c
	if (fexist) then 
          call rddat(prname,namlth,ierr)
          if (ierr.eq.-100) then
            write (luttyo,1201) prname(:namlth)
            write (lulog,1201) prname(:namlth)
            go to 110
          else if (ierr.ne.0) then
            write (luttyo,1200) prname(:namlth)
            write (lulog,1200) prname(:namlth)
            go to 110
          end if
        else
	  write (luttyo,1100) prname(:namlth)
          write (lulog,1100) prname(:namlth)
          go to 110
        endif
c
c----------------------------------------------------------------------
c     check to make sure that CG calculation desired
c----------------------------------------------------------------------
c
        if (itype.ne.1) then
          write (luttyo,5100) prname(:namlth)
          write (lulog,5100)  prname(:namlth)
          go to 110
        end if
c
c---------------------------------------------------------------------
c    print out input parameter set
c---------------------------------------------------------------------
c		
	call wrparm(prname,namlth,lulog)
c
c----------------------------------------------------------------------
c      Determine basis set indices (0: read .ind file if present)
c----------------------------------------------------------------------
c
        call lbasix( ixname,namlth,0 )
c
c---------------------------------------------------------------------
c     calculate d20k
c---------------------------------------------------------------------
c
        npts=512
c
        do i=1,npts
          spect(1,i)=zero
          spect(2,i)=zero
        enddo
c
        if (kk0.eq.0) then
           jk2mn=1
           jk2sp=1
        else
           jk2mn=-1
           jk2sp=-2
        endif
c  
        do jk2 = 1,jk2mn,jk2sp
c       do jk2 = 1,1

        write(luttyo,1040)
        write(lulog,1040)
c
        call matr2k(ierr,jk2,kk0)
c
        if (ierr.eq.0) then
          write (luttyo,1050) ndim,nelre,nelim,neltot,cpu1
          write (lulog,1050) ndim,nelre,nelim,neltot,cpu1
        else
          write (luttyo,1070) cpu1
          write (lulog,1070) cpu1
          if (ierr.eq.1) then
            write (luttyo,1080) mxel
            write (lulog,1080) mxel
          else if (ierr.eq.2) then
            write (luttyo,1090) mxdim
            write (lulog,1090) mxdim
          end if
          go to 110
        end if
c
c---------------------------------------------------------------------
c     starting vector calculation
c---------------------------------------------------------------------
c
        write (luttyo,2000)
        write (lulog,2000)
c
        do 40 i=1,mxdim
          y(1,i)=zero
          y(2,i)=zero
 40     continue
c
        call stvect(x)
c
        call scmvmd(x,y,ndim,jk2)
c
        nelv=0
c
        vnorm=zero
        do 45 i=1,ndim
          vnorm=vnorm+y(1,i)*y(1,i)
          x(1,i)=y(1,i)
          x(2,i)=y(2,i)
          y(1,i)=zero
          y(2,i)=zero
          if (abs(x(1,i)).gt.rndoff) nelv=nelv+1
 45     continue
c
        vnorm=one/dsqrt(vnorm)
        write(4,*) 'vnorm=',vnorm
        do i=1,ndim
         x(1,i)=x(1,i)*vnorm
        enddo
c
        do i=ndim+1,mxdim
          x(1,i)=zero
          x(2,i)=zero
          y(1,i)=zero
          y(2,i)=zero
        enddo
c
c........................................
c Timing call specific to UNIX systems
c       cpu2 = dtime( tarray )
c........................................
c
        write (luttyo,2010) nelv,cpu2
        write (lulog,2010) nelv,cpu2
c
c----------------------------------------------------------------------
c     remove comments to store starting vector
c----------------------------------------------------------------------
c
        if (stvflg) then
          open (unit=ludisk,file=vtname(:namlth),status='unknown',
     #           access='sequential',form='unformatted')
          rewind (unit=ludisk)
          write (ludisk) (x(1,i),x(2,i),i=1,ndim)
          close (unit=ludisk)
        end if
c
c---------------------------------------------------------------------
c     create matrix
c---------------------------------------------------------------------
c
        write(luttyo,1040)
     	write(lulog,1040) 
c
        call matrsr(ierr,jk2)
c
c........................................
c Timing call specific to UNIX systems
c       cpu1 = dtime( tarray )
c........................................
c
        if (ierr.eq.0) then
          write (luttyo,1050) ndim,nelre,nelim,neltot,cpu1
	  write (lulog,1050) ndim,nelre,nelim,neltot,cpu1
        else 
          write (luttyo,1070) cpu1
          write (lulog,1070) cpu1
          if (ierr.eq.1) then
            write (luttyo,1080) mxel
            write (lulog,1080) mxel
          else if (ierr.eq.2) then
            write (luttyo,1090) mxdim
	    write (lulog,1090) mxdim
          end if
          go to 110
        end if
c
c----------------------------------------------------------------------
c     Output matrix if desired
c----------------------------------------------------------------------
c
      if (stmflg) then
        open (unit=ludisk,file=mtname(:namlth),status='unknown',
     #        access='sequential',form='unformatted')
        rewind (unit=ludisk)
        write (luttyo,4000) mtname(:namlth)
        write (ludisk) (zdiag(1,i),zdiag(2,i),i=1,ndim)
        write (ludisk) (jzmat(i),i=1,ndim+1)
        write (ludisk) (izmat(i),zmat(i),i=1,nelim)
        write (ludisk) (kzmat(i),i=1,ndim+1)
        write (ludisk) (izmat(mxel-i+1),zmat(mxel-i+1),i=1,nelre)
        close (unit=ludisk)
      end if
c
c---------------------------------------------------------------------
c     call conjugate gradients routine
c---------------------------------------------------------------------
c
        write (luttyo,3000) 
        write (lulog,3000) 
c
        call cscg(x,ndim,nstep,cgtol,dcmplx(shiftr,shifti),
     #       y,alpha,beta,ndone,error)
c
c........................................
c Timing call specific to UNIX systems
c       cpu3 = dtime( tarray )
c........................................
c
        if (ndone.gt.0) then
          write(luttyo,3010) ndone,error,cpu3
          write(lulog,3010) ndone,error,cpu3
        else
          write(luttyo,3020) nstep,-ndone,cgtol,error,cpu3
          write(lulog,3020) nstep,-ndone,cgtol,error,cpu3
        end if
c
c----------------------------------------------------------------------
c     set nstep equal to the number of CG steps done
c----------------------------------------------------------------------
c
        nstepl=abs(ndone)
c
c---------------------------------------------------------------------
c     update parameter file
c---------------------------------------------------------------------
c
        write (luttyo,4000) prname(:namlth)
        write (lulog,4000) prname(:namlth)
c
        call wrdat(prname,namlth)
c
c---------------------------------------------------------------------
c     store tridiagonal matrix
c---------------------------------------------------------------------
c
        write (luttyo,4000) tdname(:namlth)
        write (lulog,4000) tdname(:namlth)
c
        open (unit=ludisk,file=tdname,status='unknown',
     #       access='sequential',form='unformatted')
        rewind (unit=ludisk)
        write(ludisk) (alpha(1,j),alpha(2,j),beta(1,j),beta(2,j),
     #       j=1,nstep)
        close (unit=ludisk)
c
c----------------------------------------------------------------------
c       Diagonalize tridiagonal matrix
c----------------------------------------------------------------------
c
        isort=2
        ndiag=nstep
c
        do i=1,ndiag
          temp1(1,i)=alpha(1,i)
          temp1(2,i)=alpha(2,i)
          temp2(1,i)=beta(1,i)
          temp2(2,i)=beta(2,i)
        enddo
c
c       temp2(1,ndiag)=zero
c       temp2(2,ndiag)=zero
c
        call cmtqli(ndiag,temp1,temp2,w,ierr,isort)
c
        if (ierr.ne.0) write(luttyo,1140) ierr
c
        do i=1,ndiag
          tr=w(1,i)
          ti=w(2,i)
          w(1,i)=tr*tr-ti*ti
          if (abs(w(1,i)).lt.rndoff) w(1,i)=zero
          w(2,i)=two*tr*ti
          if (abs(w(2,i)).lt.rndoff) w(2,i)=zero
c         w(1,i)=dsqrt(w(1,i))
        enddo 
c
c     *** Output the eigenvalues
c
        open (unit=ludisk,file=egname(:namlth),status='unknown',
     #       access='sequential',form='formatted')
        write (ludisk,1130) (dble(i),temp1(1,i),temp1(2,i),
     #       w(1,i),w(2,i),i=1,ndiag)
c
c----------------------------------------------------------------------
c       Initialize field array and calculate spectral density
c----------------------------------------------------------------------
c
        npts=512
c       nstepl=nstep
        ideriv=0
c
        db=(bfn-bin)/dble(npts-1)
        bsweep=bin
        do 80 i=1,npts
          spect0(1,i)=zero
          spect0(2,i)=zero
          field(i)=bsweep
          bsweep=bsweep+db
 80     continue
c 
        sd1=vnorm*vnorm
        sd1=0.25D0/sd1
c
        if (kk0.eq.0) then
           sd1=5*sd1
        else
           sd1=10*sd1
        endif
c
c       sd1=1.0D0/sd1
        write(4,*) 'normalization constant= ',sd1
c       error=jmx*(jmx+1)*drt+imx*imx*(dpt-drt)
c       error=1.0D0/error
c       write(*,*) 'error=', error
c
        do i=1,npts
          spect0(1,i)=0.0D0
          sd3=field(i)*field(i)
          do j=1,nstep
c         do j=1,10
            if (temp1(1,j).gt.rndoff) then 
c           if (temp1(1,j).gt.error) then 
c           if (w(1,j)/temp1(1,j).le.w(1,1)/temp1(1,1)) then 
              sd2=sd3+temp1(1,j)*temp1(1,j)
              spect0(1,i)=spect0(1,i)+w(1,j)*temp1(1,j)/sd2
            endif
          enddo
c         spect(1,i)=spect(1,i)+spect0(1,i)*sd1
          if (kk0.eq.0) then
             spect(1,i)=spect(1,i)+spect0(1,i)
          else
             spect(1,i)=spect(1,i)+spect0(1,i)*0.5D0
          endif
        enddo 
c
c       if (nstepl.gt.nstep) nstepl=nstep
c       do i=1,nstepl
c         temp1(1,i)=alpha(1,i)
c         temp1(2,i)=alpha(2,i)
c         temp2(1,i)=beta(1,i)
c         temp2(2,i)=beta(2,i)
c         write(*,*) temp1(1,i),temp2(1,i)
c       enddo
c
c       ideriv=0
c       call cfvd(temp1,temp2,nstepl,field,npts,wint0n,spect0,ideriv)
c
c       do i=1,npts
c         spect(1,i)=spect(1,i)+spect0(1,i)/vnorm/vnorm*0.25D0*jk2
c         spect(2,i)=spect(2,i)+spect0(2,i)/vnorm/vnorm*0.25D0*jk2
c         spect(1,i)=spect(1,i)+spect0(1,i)/vnorm/vnorm*0.25D0
c         spect(2,i)=spect(2,i)+spect0(2,i)/vnorm/vnorm*0.25D0
c       enddo
c
        enddo
c
c----------------------------------------------------------------------
c       Calculate spectral density using model free approach
c----------------------------------------------------------------------
c
        c(1)=c20
        c(2)=c22
        c(3)=c40
        c(4)=c42
        c(5)=0.0D0
c
        tauc=one/6.0D0/drt
        taum=one/6.0D0/d0rt
        if (kk0.eq.1) then
           tauc=5.0D0*drt+1.0D0*dpt
           tauc=one/tauc
        else if (kk0.eq.2) then
           tauc=2.0D0*drt+4.0D0*dpt
           tauc=one/tauc
        endif
c
        if (kk0.eq.1) then
           taum=5.0D0*d0rt+1.0D0*d0pt
           taum=one/taum
        else if (kk0.eq.2) then
           taum=2.0D0*d0rt+4.0D0*d0pt
           taum=one/taum
        endif
c
c       tau=one/tauc
c       tau=one/tauc+one/taum
c       tau=one/tau
        if (kk0.eq.0) then
c         if (abs(c20-0.0D0).lt.rndoff) orpr=0.0
c         if (abs(c20-1.0D0).lt.rndoff) orpr=0.22048
c         if (abs(c20-2.0D0).lt.rndoff) orpr=0.43928
c         if (abs(c20-3.0D0).lt.rndoff) orpr=0.60506
c         if (abs(c20-3.2D0).lt.rndoff) orpr=0.63064
c         if (abs(c20-4.0D0).lt.rndoff) orpr=0.71156
c         if (abs(c20-5.0D0).lt.rndoff) orpr=0.77715
c         if (abs(c20-9.0D0).lt.rndoff) orpr=0.88365
c         if (abs(c20-10.0D0).lt.rndoff) orpr=0.89590
          orpr=order(c)
        else
          orpr=0.0
        endif
        write(*,*) 'c20,orpr=', c(1),orpr
        sd1=orpr*orpr*taum
        sd2=(one-orpr*orpr)*tauc
c    
        do i=1,npts
          sd3=field(i)*field(i)
c         spect(2,i)=0.2D0*(sd1/(one+sd3*taum*taum)+
          spect(2,i)=(sd1/(one+sd3*taum*taum)+
     #                      sd2/(one+sd3*tauc*tauc))
        enddo
c
c----------------------------------------------------------------------
c     write spectrum to disk
c----------------------------------------------------------------------
c
          open (unit=ludisk,file=spname(:namlth),status='unknown',
     #         form='formatted')
          write (ludisk,7100)(field(i),spect(1,i),
     #                        spect(2,i),i=1,npts)
          close (unit=ludisk)
c
c----------------------------------------------------------------------
c       End of loop over input files
c----------------------------------------------------------------------
 110	continue
c
c---------------------------------------------------------------------
c     print out trailer and close log file
c---------------------------------------------------------------------
c
	write (luttyo,1000)
	write (lulog,1000)
c
	close (unit=lulog)
c
c======================================================================
c     end of loop over calculations
c======================================================================
c
c       cputot=cputot+cpu1+cpu2+cpu3
c
 100  continue
c
c---------------------------------------------------------------------
c     exit program
c---------------------------------------------------------------------
c
c9999 write (luttyo,6000) cputot
 
c9999  stop
 9999 continue
c
c=====================================================================
c     format statements
c=====================================================================
c
 1000 format(//,2x,70('#'),//)
 1010 format(25x,'PROGRAM : EPRCGS',/,25x,'----------------',//)
c
 1040 format(/,15x,'*** Calculating matrix elements ***')
 1050 format(/,2x,'Dimension of the matrix : ',i6,
     #     /,2x,'Number of off-diagonal real elements: ',i8,
     #     /,2x,'Number of off-diagonal imaginary elements: ',i8,
     #     /,2x,'Total number of off-diagonal elements: ',i8,
     #     /,2x,'CPU time (seconds) : ',f10.2,/)
 1070 format(//,10x,
     #     '*** Serious error encountered in matrix generation ***'
     #     //20x,'CPU time (seconds) : ',f10.2,/)
 1080 format(/,10x,
     #  '*** Number of off-diagonal matrix elements exceeds ',i8,' ***')
 1090 format(/,20x,'*** Matrix dimension exceeds ',i6,' ***')
 1100 format(/,20x,'*** File ''',a,''' not found ***',/)
 1200 format(/,10x,'*** Error reading file ''',a,''' ***',/)
 1201 format(/,10x,'*** ''',a,''' is not an EPRSR Version 1.4 file',
     #' ***')
c
 2000 format(/,15x,'*** Calculating starting vector ***')
 2010 format(/,2x,'Number of non-zero elements in ',
     #            'starting vector : ',i5,
     #       /,2x,'CPU time (seconds) : ',f10.2)
c
 3000 format(/,15x,'*** Calculating tridiagonal matrix ***')
 3010 format(/,2x,'Number of CG steps executed : ',i3,
     #       /,2x,'Error in solution vector : ',g14.7,
     #       /,2x,'CPU time (seconds) : ',f10.2)
 3020 format(//,10x,'*** CG CALCULATION DID NOT CONVERGE ***'
     #     /,10x,'*** MAXIMUM NUMBER OF CG STEPS : ',i3,' ***',
     #     /,10x,'*** NUMBER OF CG STEPS EXECUTED : ',i3,' ***',
     #     /,10x,'*** MAXIMUM ERROR ALLOWED : ',g14.7,' ***',
     #     /,10x,'*** FINAL ERROR  : ',g14.7,' ***', 
     #     /,10x,'*** CPU TIME (SECONDS) : ',f10.2,' ***')
c
 4000 format(/,20x,'*** Writing ',a,' ***')
 5000 format(15x,38('='),/,
     #     15x,'Calculation no. ',I2,' using file ',a,/,
     #     15x,38('='),/)
 5100 format(//,10x,
     #     '*** ''',a,''' is not set to run EPRCGS (itype=1) ***')
c
 6000 format(/,2x,'Total CPU time (seconds) : ',f10.2)
c
 7100   format(3(g14.7,','))
c
 1130   format(5(g14.7,','))
 1140   format(//,10x,'*** ERROR IN DIAGONALIZATION, IERR=',i3,' ***')
c
c=====================================================================
c
      end
