c Based on EPRLL - VERSION 1.4     4/13/93
c*********************************************************************
c
c                         ===============
c                         PROGRAM : EPRSR
c                         ===============
c                  **** Pruned basis version ***
c
c       This program does the EPR spectral calculation using the 
c       complex symmetric Lanczos algorithm, in the presence
c       of a rotating slowly relaxing local structure (SRLS).
c
c       written  by DJS  29-AUG-87
c       modified by DEB 13-MAR-92 to use longer filenames
c                       22-OCT-92 Handles case of version incompatibility
c       modified by AP  17-FEB-94 for EPRSR calculation
c
c       Includes :
c               stddim.inc
c               stdio.inc
c               eprmat.inc
c               eprdat.inc
c               timer.inc
c               fnames.inc
c
c       Uses :
c               getids.f
c               setnam.f
c               rddat.f
c               wrdat.f
c               lbasix.f
c               matrsr.f
c               stvect.f
c               cslnzs.f
c
c*********************************************************************
c
      program eprsr
c
      implicit none
c
      include 'stddim.inc'
      include 'stdio.inc'
      include 'eprdat.inc'
      include 'eprmat.inc'
      include 'fnames.inc'
      include 'timer.inc'
      include 'rndoff.inc'
      include 'pidef.inc'
c
      integer jk2
      parameter (jk2=1)
      double precision zero,vnorm
      parameter (zero=0.0D0)
c
      logical fexist
c
      double precision x,y,alpha,beta
      dimension x(2,mxdim),y(2,mxdim),alpha(2,mxstep),beta(2,mxstep)
      integer i,ifile,j,nfiles,ncalc,ierr
      character*1 chr1,chr2
c
      integer npts,nstepl,ideriv
c
      double precision ct,be,plk
      parameter (be=9.2731d-21,plk=1.05443d-27)
c
      double precision db,bsweep
      double precision field(mxpt),spect(2,mxpt),
     #       temp1(2,mxstep),temp2(2,mxstep)
c
c#####################################################################
c
c........................................
c Timing call specific to UNIX system
       cputot = dtime( tarray )
c........................................
c
      write (luttyo,1000)
      write (luttyo,1010)
c
      call getids(nfiles,mxcalc) 
      if (nfiles.eq.0) goto 9999
c
c======================================================================
c     loop over calculations
c======================================================================
c
      do 100 ncalc=1,nfiles
c
        fileid = files(ncalc)
c
c---------------------------------------------------------------------
c     construct file names and set special flags 
c---------------------------------------------------------------------
c
        call setnam
        write (*,*) flags(ncalc)
        call setflg( flags(ncalc) )
c
c----------------------------------------------------------------------
c     open log file and write out header 
c----------------------------------------------------------------------
c
        open (unit=lulog,file=rlname(:namlth),access='sequential',
     #        form='formatted')
c
        rewind (unit=lulog)
c
        write (luttyo,1000)
        write (lulog,1000)
        write (luttyo,5000) ncalc,prname(:namlth)
        write (lulog,5000) ncalc,prname(:namlth)
c
c----------------------------------------------------------------------
c     make sure that the parameter file exists
c----------------------------------------------------------------------
c
        inquire(file=prname(:namlth),exist=fexist)
c
        if (fexist) then
          call rddat(prname,namlth,ierr)
          if (ierr.eq.-100) then
            write (luttyo,1201) prname(:namlth)
            write (lulog,1201) prname(:namlth)
            go to 110
          else if (ierr.ne.0) then
            write (luttyo,1200) prname(:namlth)
            write (lulog,1200) prname(:namlth)
            go to 110
          end if
        else
          write (luttyo,1100) prname(:namlth)
          write (lulog,1100) prname(:namlth)
          go to 110
        endif
c
c----------------------------------------------------------------------
c     check to make sure that Lanczos calculation desired
c----------------------------------------------------------------------
c
        if (itype.ne.0) then
          write (luttyo,5100) prname(:namlth)
          write (lulog,5100)  prname(:namlth)
          go to 110
        end if
c
c---------------------------------------------------------------------
c	print out parameter set
c---------------------------------------------------------------------
c
	call wrparm(prname,namlth,lulog)
c
c----------------------------------------------------------------------
c      Determine basis set indices (0: read .ind file if present)
c----------------------------------------------------------------------
c
        call lbasix( ixname,namlth,0 )
c
c---------------------------------------------------------------------
c     calculate d20k
c---------------------------------------------------------------------
c
        write(luttyo,1040)
        write(lulog,1040)
c
        call matr2k(ierr,jk2)
c
        if (ierr.eq.0) then
          write (luttyo,1050) ndim,nelre,nelim,neltot,cpu1
          write (lulog,1050) ndim,nelre,nelim,neltot,cpu1
        else
          write (luttyo,1070) cpu1
          write (lulog,1070) cpu1
          if (ierr.eq.1) then
            write (luttyo,1080) mxel
            write (lulog,1080) mxel
          else if (ierr.eq.2) then
            write (luttyo,1090) mxdim
            write (lulog,1090) mxdim
          end if
          go to 110
        end if
c
c---------------------------------------------------------------------
c     starting vector calculation
c---------------------------------------------------------------------
c
        write (luttyo,2000) 
        write (lulog,2000) 
c
        do 40 i=1,mxdim
          y(1,i)=zero
          y(2,i)=zero
 40     continue
c
        call stvect(x)
c
        call scmvmd(x,y,ndim,jk2)
c
        nelv=0
c
        vnorm=zero
        do 45 i=1,mxdim
          vnorm=vnorm+y(1,i)*y(1,i)
          x(1,i)=y(1,i)
          x(2,i)=y(2,i)
          y(1,i)=zero
          y(2,i)=zero
          if (abs(x(1,i)).gt.rndoff) nelv=nelv+1
c         if (abs(x(1,i)).gt.rndoff) write(*,*) '1',i, x(1,i)
c         if (abs(x(2,i)).gt.rndoff) write(*,*) '2',i, x(2,i)
 45     continue
c
        vnorm=1/dsqrt(vnorm)
        write(*,*) 'vnorm=',vnorm
        do i=1,mxdim
          x(1,i)=x(1,i)*vnorm
          if (abs(x(1,i)).gt.rndoff) write(*,*) '1',i, x(1,i)
        enddo
c........................................
c Timing call specific to UNIX systems
        cpu2 = dtime( tarray )
c........................................
c
        write (luttyo,2010) nelv,cpu2
        write (lulog,2010) nelv,cpu2
c
c----------------------------------------------------------------------
c     Output starting vector if desired
c----------------------------------------------------------------------
c
        if (stvflg) then
          open (unit=ludisk,file=vtname(:namlth),status='unknown',
     #           access='sequential',form='unformatted')
          rewind (unit=ludisk)
          write (ludisk) (x(1,i),x(2,i),i=1,ndim)
          close (unit=ludisk)
        end if
c 
c---------------------------------------------------------------------
c     create matrix
c---------------------------------------------------------------------
c
        write(luttyo,1040)
        write(lulog,1040) 
c
        call matrsr(ierr)
c
c........................................
c Timing call specific to UNIX systems
        cpu1 = dtime( tarray )
c........................................
c
        if (ierr.eq.0) then
          write (luttyo,1050) ndim,nelre,nelim,neltot,cpu1
          write (lulog,1050) ndim,nelre,nelim,neltot,cpu1
        else 
          write (luttyo,1070) cpu1
          write (lulog,1070) cpu1
          if (ierr.eq.1) then
            write (luttyo,1080) mxel
            write (lulog,1080) mxel
          else if (ierr.eq.2) then
            write (luttyo,1090) mxdim
            write (lulog,1090) mxdim
          end if
          go to 110
        end if
c
c----------------------------------------------------------------------
c     Output matrix if desired
c----------------------------------------------------------------------
c
      if (stmflg) then
        open (unit=ludisk,file=mtname(:namlth),status='unknown',
     #        access='sequential',form='unformatted')
        rewind (unit=ludisk)
        write (ludisk) (zdiag(1,i),zdiag(2,i),i=1,ndim)
        write (ludisk) (jzmat(i),i=1,ndim+1)
        write (ludisk) (izmat(i),zmat(i),i=1,nelim)
        write (ludisk) (kzmat(i),i=1,ndim+1)
        write (ludisk) (izmat(mxel-i+1),zmat(mxel-i+1),i=1,nelre)
        close (unit=ludisk)
      end if
c
c---------------------------------------------------------------------
c     call Lanczos routine
c---------------------------------------------------------------------
c
        write (luttyo,3000) 
        write (lulog,3000)
c
        call cslnzs(x,y,nstep,ndim,alpha,beta)
c
c........................................
c Timing call specific to UNIX systems
        cpu3 = dtime( tarray )
c........................................
c
        write (luttyo,3010) cpu3
        write (lulog,3010) cpu3
c
c---------------------------------------------------------------------
c      update parameter file
c---------------------------------------------------------------------
c
        write (luttyo,4000) prname(:namlth)
        write (lulog,4000) prname(:namlth)
c
        call wrdat(prname,namlth)
c
c---------------------------------------------------------------------
c     store tridiagonal matrix
c---------------------------------------------------------------------
c
        write (luttyo,4000) tdname(:namlth)
        write (lulog,4000) tdname(:namlth)
c
        open (unit=ludisk,file=tdname(:namlth),status='unknown',
     #       access='sequential',form='unformatted')
        rewind (unit=ludisk)
        write(ludisk) (alpha(1,j),alpha(2,j),beta(1,j),beta(2,j),
     #       j=1,nstep)
        close (unit=ludisk)
c
c----------------------------------------------------------------------
c       Initialize field array and spectrum
c----------------------------------------------------------------------
c
        npts=512
        nstepl=nstep
        ideriv=0
        ct=g0*be/plk
c
        db=(bfn-bin)/dble(npts-1)
        bsweep=bin
        do 80 i=1,npts
          spect(1,i)=zero
          spect(2,i)=zero
          field(i)=bsweep
          bsweep=bsweep+db
 80     continue
c
        if (nstepl.gt.nstep) nstepl=nstep
        do i=1,nstepl
c         temp1(1,i)=alpha(1,i)*ct
c         temp1(2,i)=alpha(2,i)*ct
c         temp2(1,i)=beta(1,i)*ct
c         temp2(2,i)=beta(2,i)*ct
          temp1(1,i)=alpha(1,i)
          temp1(2,i)=alpha(2,i)
          temp2(1,i)=beta(1,i)
          temp2(2,i)=beta(2,i)
          write(*,*) temp1(1,i),temp2(1,i)
        enddo
          write(*,*) temp1(1,2)/
     #              (temp1(1,1)*temp1(1,2)-temp2(1,1)*temp2(1,1)) 
          write(*,*) 1.0D0/(temp1(1,1)+(temp2(1,1)*temp2(1,1)/
     #               (temp1(1,2)*temp1(1,2))))
c
        ideriv=0       
        call cfvd(temp1,temp2,nstepl,field,npts,wint0n,spect,ideriv)
c
        do i=1,npts
          spect(1,i)=spect(1,i)
          spect(2,i)=spect(2,i)
        enddo
c
c----------------------------------------------------------------------
c     write spectrum to disk
c----------------------------------------------------------------------
c
          open (unit=ludisk,file=spname(:namlth),status='unknown',
     #         form='formatted')
          write (ludisk,7100)(field(i),spect(1,i),
     #                        spect(2,i),i=1,npts)
          close (unit=ludisk)
c
c----------------------------------------------------------------------
c     end of loop over calculations
c----------------------------------------------------------------------
c
 110  continue
c
c---------------------------------------------------------------------
c     print out trailer and close log file
c---------------------------------------------------------------------
c
      write (luttyo,1000)
      write (lulog,1000)
c
      close (unit=lulog)
c
c
      cputot = cputot+cpu1+cpu2+cpu3
c
 100  continue
c
c---------------------------------------------------------------------
c     exit program
c---------------------------------------------------------------------
c
 9999 write (luttyo,6000) cputot
c
      stop
c
c=====================================================================
c     format statements
c=====================================================================
c
 1000 format(//,2x,70('#'),//)
 1010 format(25x,'PROGRAM : EPRSR',/,25x,'---------------',//)
c
 1040 format(/,15x,'*** Calculating matrix elements ***')
 1050 format(/,2x,'Dimension of the matrix : ',i6,
     #     /,2x,'Number of off-diagonal real elements: ',i8,
     #     /,2x,'Number of off-diagonal imaginary elements: ',i8,
     #     /,2x,'Total number of off-diagonal elements: ',i8,
     #     /,2x,'CPU time (seconds) : ',f10.2,/)
 1070 format(//,10x,
     #     '*** Serious error encountered in matrix generation ***'
     #     //20x,'CPU time (seconds) : ',f10.2,/)
 1080 format(/,10x,
     #  '*** Number of off-diagonal matrix elements exceeds ',i8,' ***')
 1090 format(/,20x,'*** Matrix dimension exceeds ',i6,' ***')
 1100 format(/,20x,'*** File ''',a,''' not found ***',/)
 1200 format(/,10x,'*** Error reading file ''',a,''' ***',/)
 1201 format(/,10x,'*** ''',a,''' is not an EPRSR Version 1.4 file',
     #' ***')
 2000 format(/,15x,'*** Calculating starting vector ***')
 2010 format(/,2x,'Number of non-zero elements in ',
     #     'starting vector : ',i5,
     #     /,2x,'CPU time (seconds) : ',f10.2)
c
 3000 format(/,15x,'*** Calculating tridiagonal matrix ***')
 3010 format(/,2x,'CPU time (seconds) : ',f10.2)
c
 4000 format(/,20x,'*** Writing ',a,' ***')
 5000 format(15x,38('='),/,
     #     15x,'Calculation no. ',I2,' using file ',a,/,
     #     15x,38('='),/)
 5100 format(//,10x,
     #     '*** ''',a,''' is not set for EPRSR (itype=0) ***')
 6000 format(/,15x,'*** Total CPU time (seconds) : ',f10.2,' ***',//)
c
 7100   format(3(g14.7,','))
c
c=====================================================================
c
      end
