c     Version 1.4     3/13/92
c**********************************************************************
c                       ===================
c                       SUBROUTINE : LBASIX
c                       ===================
c
c       This routine builds a list of the basis set indices in common 
c       block /indexl/ for the truncation parameters set in /eprdat/ 
c       or optionally reads them from the index file "<name>.ind"
c       which contains a pruned basis set produced by the EPRBL program. 
c       If one of the following conditions is true:
c         (1) The "new" flag is set nonzero on input
c         (2) <name>.ind does not exist 
c       if the file is not
c       found, the routine builds the full basis set indicated
c       by the MTS parameters.
c
c       Includes:
c               stddim.inc
c               stdio.inc
c               eprdat.inc
c               indexl.inc
c               srsflg.inc
c
c       Uses:
c               ipar.f
c
c######################################################################
c
c See accompanying text for complete and accurate definition of
c the symmetrized basis set. Note that
c
c mr .ge. 0
c kr .ge. 0
c 
c Particular cases mr=nr=ipnr=0   --->  imr=ipar(lr+jr)
c                  mr=nr=0        --->  ipnr .ge. 0
c                  mr=0           --->  nr   .ge. 0
c
c######################################################################
c
c**********************************************************************
c
      subroutine lbasix( ixname,lth,new )
      implicit none
      character*30 ixname
      integer lth,new,i
c
      include 'stddim.inc'
      include 'stdio.inc'
      include 'eprdat.inc'
      include 'indexl.inc'
      include 'srsflg.inc'
c
      integer lr,mr,mrmx,kr,krmx,jrmn,
     #        jr,nr,nrmn,nrmx,ir,irmx,
     #        ipsr,iqsr,ipnr,iqnr,
     #        ipnrmn,ipnrmx,iqnrmn,iqnrmx,
     #        ipsrmn,ipsrmx,iqsrmn,iqsrmx,
     #        imr,imrmn,imrmx,imstep,
     #        nrow
c
      double precision zero
      parameter (zero=0.d0)
c
      logical fexist
      integer ipar
      external ipar
c
c######################################################################
c
      if (new.eq.0) then
c
c----------------------------------------------------------------------
c     Check whether the basis set indices may be found in a file
c----------------------------------------------------------------------
c
         inquire(file=ixname(:lth),exist=fexist)
         if (fexist) then
c
c----------------------------------------------------------------------
c     Read basis set information from index file
c----------------------------------------------------------------------
c
            open (unit=ludisk,file=ixname(:lth),status='old',
     #           access='sequential',form='unformatted',err=9)
            read (ludisk,err=9) ndim,lemx,lomx,mmx,kmx,
     #                          jmx,nmx,imx,ipnmx
            read (ludisk,err=9) (l1(i),m1(i),k1(i),j1(i),n1(i),
     #            i1(i),pi1(i),im1(i),qi1(i),ps1(i),qs1(i),
     #           i=1,ndim)
            close (unit=ludisk)
            return
c
c Error return: close file and continue to generate indices
c
 9          close (unit=ludisk)
         end if
c
      end if
c
c######################################################################
c
      nrow=0
c
c----------------------------------------------------------------------
c     *** loop over lr ***
c----------------------------------------------------------------------
c
      do 100 lr=0,lemx,1
        if((1-2*mod(abs(lr),2).ne.1).and.(lr.gt.lomx)) go to 100
c       if(lr.eq.1) go to 100
c
c----------------------------------------------------------------------
c     *** loop over mr ***
c----------------------------------------------------------------------
c
       if(lkmtrc.eq.1) then
           mrmx=min(mmx,lr,lemx-lr+lkmdff)
       else
           mrmx=min(mmx,lr)
       end if
       do 110 mr=0,mrmx,1
c
c----------------------------------------------------------------------
c     *** loop over kr ***
c----------------------------------------------------------------------
c
        if(lkmtrc.eq.1) then
           krmx=min(kmx,lr,lemx-lr-mr+lkmdff)
        else
           krmx=min(kmx,lr)
        end if
        do 120 kr=0,krmx,kdelta
           if (kr.eq.0.and.ipar(lr).ne.1) go to 120
c 
c----------------------------------------------------------------------
c     *** loop over jr ***
c----------------------------------------------------------------------
c
         if (lr.eq.0) then
           jrmn=0
         else
           jrmn=0
         endif 
         do 122 jr=jrmn,jmx,jdelta
c
c----------------------------------------------------------------------
c     *** loop over nr ***
c----------------------------------------------------------------------
c
          nrmx=min(nmx,jr)
c
          if (mr.eq.0) then
             nrmn=0
          else
             nrmn=-nrmx
          end if
          do 124 nr=nrmn,nrmx,1
c
c----------------------------------------------------------------------
c     *** loop over ir ***
c----------------------------------------------------------------------
c
c       if(lkmtrc.eq.1) then
c          irmx=min(imx,jr,jmx-jr+lkmdff)
c       else
           irmx=min(imx,jr)
c       end if
          do 125 ir=0,irmx
           if (ir.eq.0.and.ipar(jr).ne.1) go to 125
c 
c----------------------------------------------------------------------
c     *** loop over ipnr ***
c----------------------------------------------------------------------
c
           ipnrmx=0
           ipnrmn=0
           do 126 ipnr=ipnrmn,ipnrmx,1
              if (psi.eq.zero.and.(mr+nr).ne.ipnr) go to 126
c
c----------------------------------------------------------------------
c     *** loop over im ***
c----------------------------------------------------------------------
c
              imrmx=1
              if (hfdflg) then
                 imrmn=-1
              else
                 imrmn=1
              end if
c
            do 127 imr=imrmn,imrmx,2
               if (mr.eq.0.and.nr.eq.0.and.ipnr.eq.0.and.
     #             ipar(lr+jr).ne.imr) go to 127
c
c----------------------------------------------------------------------
c     *** loop over iqnr ***
c----------------------------------------------------------------------
c
             iqnrmx=0
             iqnrmn=0
             do 128 iqnr=iqnrmn,iqnrmx,2
c
c----------------------------------------------------------------------
c     *** loop over ipsr ***
c----------------------------------------------------------------------
c
              ipsrmx=0
              if (hfdflg) then
                 ipsrmn=0
              else
                 ipsrmn=0
              end if
c
              do 130 ipsr=ipsrmn,ipsrmx,1
c
c----------------------------------------------------------------------
c     *** loop over iqsr ***
c----------------------------------------------------------------------
c
               iqsrmx=0
               iqsrmn=0
               do 140 iqsr=iqsrmn,iqsrmx,2
c
                nrow=nrow+1
                if (nrow.le.mxdim) then
                  l1(nrow)=lr
                  m1(nrow)=mr
                  k1(nrow)=kr
                  j1(nrow)=jr
                  n1(nrow)=nr
                  i1(nrow)=ir
                  pi1(nrow)=ipnr
                  im1(nrow)=imr
                  qi1(nrow)=iqnr
                  ps1(nrow)=ipsr
                  qs1(nrow)=iqsr
                end if
c
c----------------------------------------------------------------------
c     end loop over rows                                      
c----------------------------------------------------------------------
c
 140           continue
 130          continue
 128         continue
 127        continue
 126       continue
 125       continue
 124      continue
 122     continue
 120    continue
 110   continue
 100  continue
c
      ndim=nrow
      return
      end















