c**********************************************************************
c       
c                       ==============
c                       PROGRAM : LBSR
c                       ==============
c
c       This program will prompt the user to enter the parameters 
c       required as input to the EPRSR spectral calculation program.
c      
c       modified by AP 17-FEB-94, based on LBLL written by DJS 15-NOV-87
c       and modified by DEB 15-MAR-92 to use new filename convention
c
c       Includes:
c               stdio.inc
c               rndoff.inc
c               eprdat.inc
c               maxl.inc
c               stddim.inc
c               fnames.inc
c               indexl.inc
c               srsdat.inc
c
c       Uses:
c               rddat.f
c               wrdat.f
c               lbasix.f
c               wrlbas.f
c               wrparm.f
c
c       Note:   the scaling of the diffusion coefficients (and of the
c               Heisenberg frequency) is done here, together with
c               the definition of d2psi and d2bed matrices.
c
c**********************************************************************
c
      program lbsr
c
      implicit none
c
      include 'stddim.inc'
      include 'maxl.inc'
      include 'stdio.inc'
      include 'eprdat.inc'
      include 'fnames.inc'
      include 'indexl.inc'
      include 'rndoff.inc'
c
      include 'srsdat.inc'
c
      logical fprmpt,fexist,flagav
      logical check1,check2,check4
      integer i,ierr,ifile,j,k,l,n,icopy,listb,nfiles,
     #        inlemx,inlomx,inmmx,inkmx,injmx,innmx,inimx,inpnmx
      character*80 line
c
      double precision zero,half,one,dsq13,dsq23,be,plk,ct
      parameter (zero=0.0D0,half=0.5D0,one=1.0D0)
      parameter (be=9.2731d-21,plk=1.05443d-27)
c
      integer itoken
      double precision ftoken
      logical prmpt
      external prmpt,itoken,ftoken
c
c######################################################################
c
      dsq13=dsqrt(1.0D0/3.0D0)
      dsq23=dsqrt(2.0D0/3.0D0)
c
      write (luttyo,1000)
      write (luttyo,1010) 
c
      call getids(nfiles,mxcalc)
      if (nfiles.eq.0) goto 9999
      ifile=0
c
 1    ifile=ifile+1
      if (ifile.gt.nfiles) goto 9999
      fileid=files(ifile)
      call setnam
c
c----------------------------------------------------------------------
c     check for existence and open parameter file
c----------------------------------------------------------------------
c
      inquire(file=prname,exist=fexist)
      if (fexist) then
        write (luttyo,1040) prname(:namlth)
        call rddat(prname,namlth,ierr)
        if (ierr.eq.-100) then
          write (luttyo,1041) prname(:namlth)
        else if (ierr.lt.0) then
          write (luttyo,1042) prname(:namlth)
        end if
      else
        write (luttyo,1050) prname(:namlth)
      end if
c
c----------------------------------------------------------------------
c     initialize spherical tensors
c----------------------------------------------------------------------
c
      do 10 i=1,5
        fgm(i)=zero
        fam(i)=zero
 10   continue
c
c----------------------------------------------------------------------
c     get g-tensor
c----------------------------------------------------------------------
c
c11    write (luttyo,2000) gxx,gyy,gzz
c      fprmpt=prmpt( line )
c      if (fprmpt) then
c       gxx=ftoken( line )
c       gyy=ftoken( line )
c       gzz=ftoken( line )
c      end if

c     if ( abs(gxx) .lt. rndoff  .or.
c    #     abs(gyy) .lt. rndoff  .or.
c    #     abs(gzz) .lt. rndoff) go to 11
c
c----------------------------------------------------------------------
c     get nuclear spin and A-tensor (in gauss)
c----------------------------------------------------------------------
c
c     write (luttyo,2010) in2
c     fprmpt=prmpt( line )
c     if (fprmpt) in2=itoken( line )
c     if (in2.le.0) then
c       in2=0
c     else
c       write (luttyo,2020) axx,ayy,azz
c       fprmpt=prmpt( line )
c       if (fprmpt) then
c         axx=ftoken( line )
c         ayy=ftoken( line )
c         azz=ftoken( line )
c       end if
c
c     end if
c
c----------------------------------------------------------------------
c     get center field (in gauss)
c----------------------------------------------------------------------
c
c20   write (luttyo,2030) b0
c     fprmpt=prmpt( line )
c     if (fprmpt) b0=ftoken( line )
c
c     if (b0 .lt. zero) b0=abs(b0)
c     if (b0.lt.rndoff) go to 20
c     if ( (in2 .le. 0) .or.
c    #    (dabs(axx) .lt. rndoff .and. 
c    #     dabs(ayy) .lt. rndoff .and.
c    #     dabs(azz) .lt. rndoff)     ) then
c       in2=0
c       axx=zero
c       ayy=zero
c       azz=zero
c     endif
c
c----------------------------------------------------------------------
c     calculate spherical components of F_g and F_A
c----------------------------------------------------------------------
c
      g0=(gxx+gyy+gzz)/3.0D0
      fgm(1)=half*(gxx-gyy)*(b0/g0)
      fgm(3)=dsq23*(gzz-half*(gxx+gyy))*(b0/g0)
      fgm(5)=fgm(1)
c
      a0=(axx+ayy+azz)/3.0D0
      fam(1)=half*(axx-ayy)
      fam(3)=dsq23*(azz-half*(axx+ayy))
      fam(5)=fam(1)
c
      ct=g0*be/plk
c
c----------------------------------------------------------------------
c     get probe diffusion tensor (per second)
c----------------------------------------------------------------------
c
      write (luttyo,2050) dr,dp
      fprmpt=prmpt( line )
      if (fprmpt) then
        dr=ftoken( line )
        dp=ftoken( line )
      end if
c
c----------------------------------------------------------------------
c     get SRLS diffusion coeff. (per second)
c----------------------------------------------------------------------
c
      write (luttyo,2055) d0r,d0p
      fprmpt=prmpt( line )
      if (fprmpt) then
        d0r=ftoken( line )
        d0p=ftoken( line )
      end if
c
c----------------------------------------------------------------------
c     get Heisenberg spin exchange rate (should be positive)
c----------------------------------------------------------------------
c
c      write(luttyo,2100) oss
c      fprmpt=prmpt( line )
c      if (fprmpt) oss=abs(ftoken( line ))
c      if (oss.lt.rndoff) oss=zero
c
c----------------------------------------------------------------------
c     rescale diffusion tensors and Heisemberg frequency 
c----------------------------------------------------------------------
c
c     drt=dr/ct
c     dpt=dp/ct
c     d0rt=d0r/ct
c     d0pt=d0p/ct
      drt=dr
      dpt=dp
      d0rt=d0r
      d0pt=d0p
c
c      osst=oss/ct
c
c----------------------------------------------------------------------
c     get probe potential parameters
c----------------------------------------------------------------------
c
      write (luttyo,3060)
      write (luttyo,3070) a20,a22
      fprmpt=prmpt( line )
      if (fprmpt) then
        a20=ftoken (line )
        a22=ftoken (line )
      end if
c     write (luttyo,3080) a40,a42
c     fprmpt=prmpt( line )
c     if (fprmpt) then
c       a40=ftoken(line )
c       a42=ftoken(line )
c     end if
c
c----------------------------------------------------------------------
c     get cage potential parameters
c----------------------------------------------------------------------
c
c     write (luttyo,3040)
c     write (luttyo,3050) b10,b20
c     fprmpt=prmpt( line )
c     if (fprmpt) then
c       b10=ftoken (line )
c       b20=ftoken (line )
c     end if
c
c----------------------------------------------------------------------
c     get interation potential parameters
c----------------------------------------------------------------------
c
      write (luttyo,3010)
      write (luttyo,3020) c20,c22
      fprmpt=prmpt( line )
      if (fprmpt) then
        c20=ftoken (line )
        c22=ftoken (line )
      end if
c     write (luttyo,3030) c40,c42
c     fprmpt=prmpt( line )
c     if (fprmpt) then
c       c40=ftoken(line )
c       c42=ftoken(line )
c     end if
c
c----------------------------------------------------------------------
c     get director tilt
c----------------------------------------------------------------------
c
c     write(luttyo,2140) psi
c     fprmpt=prmpt( line )
c     if (fprmpt) psi=ftoken( line )
c     call caldlm(d2psi,psi)
c
c----------------------------------------------------------------------
c     get cage tilt
c----------------------------------------------------------------------
c
      write(luttyo,2150) bag
      fprmpt=prmpt( line )
      if (fprmpt) bag=ftoken( line )
c
c----------------------------------------------------------------------
c     get diffusion tilt
c----------------------------------------------------------------------
c
      write(luttyo,2160) bed
      fprmpt=prmpt( line )
      if (fprmpt) bed=ftoken( line )
      call caldlm(d2bed,bed)
c
c----------------------------------------------------------------------
c     Get basis set truncation parameters
c----------------------------------------------------------------------
c
 272  write(luttyo,2170) lemx,lomx,mmx,kmx,jmx,nmx,imx,ipnmx
      fprmpt=prmpt( line )
      if (fprmpt) then
        lemx=itoken( line )
        lomx=itoken( line )
        mmx=itoken( line )
        kmx=itoken( line )
        jmx=itoken( line )
        nmx=itoken( line )
        imx=itoken( line )
        ipnmx=itoken( line )
      end if
c
c     lkmtrc=0
c     write (luttyo,2172) lkmtrc
c     fprmpt=prmpt( line )
c     if (fprmpt) lkmtrc=itoken( line )
c     if (lkmtrc.ne.0) then
c        lkmtrc=1
c        lkmdff=0
c        write(luttyo,2173) lkmdff
c        fprmpt=prmpt( line )
c        if (fprmpt) lkmdff=itoken( line )
c     else
c        lkmdff=0
c     end if
c
c----------------------------------------------------------------------
c     Check basis set parameters
c----------------------------------------------------------------------
      inlemx =lemx
      inlomx =lomx
      inmmx =mmx
      inkmx =kmx
      injmx =jmx
      innmx =nmx
      inimx =imx
      inpnmx=ipnmx
c
      if(lemx.lt.0) lemx=0
      if(mmx.lt.0) mmx=0
      if(mmx.gt.lemx) mmx=lemx
      if(kmx.lt.0) kmx=0
      if(kmx.gt.lemx) kmx=lemx
c
      if((lemx.gt.mxlval)) then
         write (luttyo,5008) mxlval
         lemx=mxlval
      endif
c
      if((jmx.gt.mxlval)) then
         write (luttyo,5008) mxlval
         jmx=mxlval
      endif
c
      if (b10.eq.zero) then
         jdelta=2
      else
         jdelta=1
      end if
c
      if(ipnmx.lt.0) ipnmx=0
      if(ipnmx.gt.in2) ipnmx=in2
c
c----------------------------------------------------------------------
c     Report any changes in basis truncation parameters
c----------------------------------------------------------------------
      if (inlemx .ne. lemx .or.
     #    inlomx .ne. lomx .or.
     #    inmmx .ne. mmx .or.
     #    inkmx  .ne. kmx  .or.
     #    injmx  .ne. jmx  .or.
     #    innmx  .ne. nmx  .or.
     #    inimx  .ne. imx  .or.
     #    inpnmx .ne. ipnmx)
     #    write (luttyo,5009) lemx,lomx,mmx,kmx,jmx,nmx,imx,ipnmx
c
c     if (ndim.gt.mxdim) then
c       write (luttyo,5011) mxdim
c       goto 272
c     end if
c
c----------------------------------------------------------------------
c     get number of Lanczos/CG steps to execute and
c     the maximum allowed error for CG
c----------------------------------------------------------------------
c
      if (nstep.gt.ndim) nstep=ndim
      write(luttyo,2180) nstep
      fprmpt=prmpt( line )
      if (fprmpt) nstep=itoken( line )
      if (nstep.gt.mxstep) nstep=mxstep
c
      write (luttyo,2190) itype
      fprmpt=prmpt( line )
      if (fprmpt) itype=itoken( line )
      if (itype.lt.0) then
        itype=0
      end if
c
      if (itype.ge.1) then
        write (luttyo,2200) cgtol
        fprmpt=prmpt( line )
        if (fprmpt) cgtol=ftoken( line )
c       if (cgtol.lt.10.0D0*rndoff) cgtol=10.0D0*rndoff
c
        write (luttyo,2210) shiftr,shifti
        fprmpt=prmpt( line )
        if (fprmpt) then
          shiftr=ftoken( line )
          shifti=ftoken( line )
        end if
        if (shiftr.lt.10.0D0*rndoff) shiftr=10.0D0*rndoff
      else
        shiftr=zero 
        shifti=zero 
      end if
c
c     Choose a field range and number of fields for field-swept
c     calculation
c
      if (itype.ge.2) then
        write (luttyo,2220) b0,fieldi,fieldf
        fprmpt=prmpt( line )
        if (fprmpt) then
          fieldi=ftoken( line )
          fieldf=ftoken( line )
        end if
c
        write (luttyo,2230) mxcalc+1,nfield
        fprmpt=prmpt( line )
        if (fprmpt) nfield=itoken( line )
        if (itype.eq.2 .and. nfield.gt.mxcalc) nfield=mxcalc
        if (itype.gt.2 .and. nfield.gt.mxpt) nfield=mxpt
c       if (nfield.lt.2) nfield=2
      end if
c
c     Set pruning tolerance for EPRBL calculation
c
      if (itype.ge.2) then
         write (luttyo,2270) btol
         fprmpt=prmpt ( line )
         if (fprmpt) btol=ftoken( line )
         if (btol.lt.zero) btol=zero
         if (btol.gt.half) btol=half
      end if
c
c----------------------------------------------------------------------
c                       2
c     input k value in D
c                       0,k
c----------------------------------------------------------------------
c
c     kk0=0
      write (luttyo,6100) kk01,kk02
      fprmpt=prmpt ( line )
      if (fprmpt) kk01=ftoken( line )
      if (fprmpt) kk02=ftoken( line )
c
c     if ((kk0.eq.0).and.(bed.eq.zero)) then
c        kdelta=2
c     else
         kdelta=1
c     end if
c
      call lbasix(ixname,namlth,0) 
      write(luttyo,5010) ndim
c
c----------------------------------------------------------------------
c     get initial and final field
c----------------------------------------------------------------------
c
c     bfn=0.0e0
c     bfn=6.00e11
      write(luttyo,6200) bin,bfn
      fprmpt=prmpt(line)
      if (fprmpt) then
        bin=ftoken(line)
        bfn=ftoken(line)
      end if
c
c----------------------------------------------------------------------
c     get 1/T20 and 1/T22
c----------------------------------------------------------------------
c
      wint0n=zero
      wint2n=zero
      write(luttyo,6300) wint0n,wint2n
      fprmpt=prmpt(line)
      if (fprmpt) then
        wint0n=ftoken(line)
        wint2n=ftoken(line)
       end if
c
c----------------------------------------------------------------------
c     print out new values of parameters
c----------------------------------------------------------------------
c
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c     ordering of potential parameters and evaluation of F expansion;
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c
c      call intfc
c
c      write (luttyo,9000) '# of U-coef. : ',idmx
c      write (luttyo,9010)
c      do i=1,idmx
c         write (luttyo,9020) i,ll(i),mm(i),kk(i),jj(i),nn(i),cc(i)
c      end do
c
c      call cfpot
c
c      write (luttyo,9000) '# of F-coef. : ',idfx
c      write (luttyo,9010)
c      do i=1,idfx
c         write (luttyo,9020) i,lf(i),mf(i),kf(i),jf(i),nf(i),cf(i)
c      end do
c
c 9000 format(//,2x,a16,i3)
c 9010 format(//,4x,'i',5x,'l',5x,'m',5x,'k',5x,'j',5x,'n',5x,'c',/)
c 9020 format(2x,6(i3,3x),g12.6)
c
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c
      call wrparm(prname,namlth,luttyo) 
c
      listb=0
      write(luttyo,4000)
      read (luttyi,4001) line
      if (line(1:1).eq.'y' .or. line(1:1).eq.'Y') listb=1
c
c----------------------------------------------------------------------
c     write parameters to formatted disk file
c----------------------------------------------------------------------
c
      open (unit=lulog,file=fmname,status='unknown',
     #      access='sequential',form='formatted')
      rewind (unit=lulog)
c
      call wrparm(prname,namlth,lulog)
c
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c
c      write (lulog,9000) '# of U-coef. : ',idmx
c      write (lulog,9010)
c      do i=1,idmx
c         write (lulog,9020) i,ll(i),mm(i),kk(i),jj(i),nn(i),cc(i)
c      end do
c
c      write (lulog,9000) '# of F-coef. : ',idfx
c      write (lulog,9010)
c      do i=1,idfx
c         write (lulog,9020) i,lf(i),mf(i),kf(i),jf(i),nf(i),cf(i)
c      end do
c
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c
c      do i=1,ndim
c         write (*,1234) l1(i),m1(i),k1(i)
c 1234    format(2x,/,3(i2,','))
c      end do

      if (listb.eq.1) call wrlbas(prname,namlth,lulog)
c
      close (unit=lulog)
c
c----------------------------------------------------------------------
c     write parameters to binary disk file 
c----------------------------------------------------------------------
c
      call wrdat(prname,namlth)
      go to 1
c
 9999 write(luttyo,1000)
      stop
c
c======================================================================
c               format statements
c======================================================================
c
 1000 format(//,2x,70('#'),//)
 1010 format(25x,'PROGRAM : LBSR',/,25x,'--------------',//)
 1040 format(2x,'Data file ',a,' already exists on the disk')
 1041 format(2x,'*** ''',a,''' is not an EPRSR Version .PAR file',
     #' ***')
 1042 format(2x,'*** Error reading file ''',a,''' ***')
 1050 format(2x,'Data file ',a,' not found')
c
 2000 format(2x,'g-tensor [gxx,gyy,gzz] : ',3(f10.7,2x))
 2010 format(2x,'twice the nuclear spin [in2] : ',i2)
 2020 format(2x,'A-tensor [axx,ayy,azz] (gauss) : ',3(f9.3,2x))
 2030 format(2x,'static field [B0] (gauss) : ',f10.3)
c
 2050 format(2x,'diffusion tensor    [dr,dp] (1/sec) =',2(g11.4,2x))
 2055 format(2x,'SRLS  diffusion [d0r,d0p] (1/sec) =',2(g11.4,2x))
c
 2100 format(2x,'Heisenberg spin exchange frequency [oss] = ',g11.4)
c
 3060 format (2x,'coefficients of the probe potential : ')
 3070 format (2x,'rank = 2   ... a20, a22      = ',2(g11.4,2x))
 3080 format (2x,'rank = 4   ... a40, a42      = ',2(g11.4,2x)) 
 3040 format (2x,'coefficients of the cage potential : ')
 3050 format (2x,'rank = 1,2 ... b10, b20      = ',2(g11.4,2x))
 3010 format (2x,'coefficients of the interation potential : ')
 3020 format (2x,'rank = 2   ... c20, c22      = ',2(g11.4,2x))
 3030 format (2x,'rank = 4   ... c40, c42      = ',2(g11.4,2x)) 
c
 2140 format(2x,'angle between B0 and local director [psi] ',
     #     '(degrees) : ',f7.2)
 2150 format(2x,'cage tilt [bag] (degrees) = ',f7.2)
 2160 format(2x,'diffusion tilt [bed] (degrees) = ',f7.2)
c
 2170 format(2x,'truncation values  [lemx,lomx,mmx,kmx,jmx,nmx,'
     #     ,'imx,ipnmx] : ',8(I2,2x))
c
 2172 format(2x,'L+K+M truncation rule flag [lkmtrc] : ',i2)
 2173 format(2x,'L+K+M truncation bandwidth [lkmdff] : ',i3)
c
 2180 format(2x,'number of Lanczos/CG steps [nstep] : ',i5)
 2190 format(2x,'calculation type (0=L,1=CG,2=FS) [itype] : ',i1)
 2200 format(2x,'error tolerance for residual [cgtol] : ',g11.4)
 2210 format(2x,'origin shift [shiftr,shifti] :',2(2x,g11.4))
 2220 format(2x,'initial and final fields relative to B0=', f10.3,
     #     /,2x,'[fieldi,fieldf] (gauss) : ',f10.3,2x,f10.3)
 2230 format(2x,'number of field positions [nfield<',i3,'] : ',i3)
 2270 format(2x,'pruning tolerance [btol] : ',g9.3)
c
 6100 format(2x,'k1 and k2 in the Wigner rotational matrix', 
     #     /,2x '[kk01,kk02] : ',2i2)
 6200 format(2x,'initial and final fields [bin,bfn] :',2(2x,g11.4))
 6300 format(2x,'1/T20 and 1/T22 [wint0n,wint2n] :',2(2x,g11.4))
c
 4000 format(2x,'List basis set indices? [N]: ')
 4001 format(a)
 5008 format(//,'lemx must be ',i4,' or less')
 5009 format(' ** Basis set parameters adjusted to [lemx,lomx,mmx,
     #       kmx,','jmx,nmx,imx,ipnmx]  = ',/,10x,8(i2,','),' **')
 5010 format(/,2x,'*** Dimension of basis = ',i7,' ***',/)
 5011 format(/,18x,'************* WARNING ****************',
     #       /,21x,   'BASIS DIMENSION EXCEEDS ',i7/
     #       /,18x,'**************************************',//
     #     2x,'Please respecify basis truncation parameters'/)
c
c======================================================================
c
      end

