c**********************************************************************
c                        =========================
c                           subroutine MATR2K
c                        =========================
c
c       Subroutine for calculating matrix elements for EPRSR calculation.
c       Based on MATRLL, written by AP 17-FEB-94 for EPRSR calculation.
c
c       Includes :
c               stddim.inc
c               rndoff.inc
c               eprdat.inc
c               eprmat.inc
c               maxl.inc
c               srsdat.inc
c               srsflg.inc
c               srsind.inc
c               srssto.inc
c
c       Uses:
c               caldlm.f
c               ipar.f
c               w3j.f
c
c**********************************************************************
c
      subroutine matr2k(ierr,jk2,kk0a1,kk0a2,kki)
c
      implicit none
c
      include 'stddim.inc'
      include 'stdio.inc'
      include 'rndoff.inc'
      include 'eprmat.inc'
      include 'eprdat.inc'
      include 'indexl.inc'
      include 'maxl.inc'
      include 'srsdat.inc'
      include 'srsflg.inc'
      include 'srsind.inc'
c     include 'srssto.inc'
c
      integer jk2,kk0a1,kk0a2,ipkk01,ipkk02,kki
      integer i,j,is,it,isa,isb,ikr,ikc,ih,imr,imc
      integer ipp,jin,jfi,iskr,iskc,ismr,ismc,iir,iic,isir,isic
      integer neli,nel,ierr,nelr
      integer nrow,ncol
      integer num,lt1,lt2,lt3
c
      double precision zero,cliou,cgam,cgamf,factr,factc
      double precision c,fact,d2
c
      logical check,fdiff,fliou,delta
c
      parameter (zero=0.d0,d2=0.707106781d0)
c
      double precision w3j
      integer ipar 
c
c######################################################################
c
      if (ndim.gt.mxdim) then
        ierr=2
        go to 9999
      end if
c
      write(*,*) 'jk2=',jk2
c
c----------------------------------------------------------------------
c     Initialize counters
c----------------------------------------------------------------------
c
      nelr=0
      neli=0
      nel=0
c
c----------------------------------------------------------------------
c     **** Loop over rows ***
c----------------------------------------------------------------------
c
      do 400 nrow=1,ndim
        lr=l1(nrow)
        mr=m1(nrow)
        kr=k1(nrow)
        jr=j1(nrow)
        nr=n1(nrow)
        ir=i1(nrow)
        ipnr=pi1(nrow)
        imr=im1(nrow)
        iqnr=qi1(nrow)
        ipsr=ps1(nrow)
        iqsr=qs1(nrow)
c
        jzmat(nrow)=neli+1
        kzmat(nrow)=nelr+1
c
c.... find normalization factor
c
          if (kr.eq.0) then
             factr=0.5d0
             ikr=ipar(lr)
          else
             factr=d2
             ikr=jk2
          end if
c
c----------------------------------------------------------------------
c   **** Loop over columns (note only upper diagonal) ****
c----------------------------------------------------------------------
c
        do 300 ncol=1,ndim
          lc=l1(ncol)
          mc=m1(ncol)
          kc=k1(ncol)
          jc=j1(ncol)
          nc=n1(ncol)
          ic=i1(ncol)
          ipnc=pi1(ncol)
          imc=im1(ncol)
          iqnc=qi1(ncol)
          ipsc=ps1(ncol)
          iqsc=qs1(ncol)
c
c.... find normalization factor
c
          if (kc.eq.0) then
             factc=0.5d0
             ikc=ipar(lc)
          else
             factc=d2
             ikc=1
          end if
c
          fact=factr*factc
          fact=fact*sqrt(dble((2*lr+1)*(2*lc+1)))
     #         *ipar(kr+mr)*w3j(lr,2,lc,-mr,0,mr)*(1+ikr*ikc*jk2) 
c
c.... find parities of the symmetrized elements .......................
c
          iskr=ipar(lr+kr)
          isir=ipar(jr+ir)
          ismr=ipar(lr+jr+mr+nr)
          iskc=ipar(lc+kc)
          isic=ipar(jc+ic)
          ismc=ipar(lc+jc+mc+nc)
          ipkk01=ipar(kk0a1)
          ipkk02=ipar(kk0a2)
c
c----------------------------------------------------------------------
c     d20k
c----------------------------------------------------------------------
c
          cgam=zero
c
          check=jr.eq.jc.and.nr.eq.nc.and.ir.eq.ic.and.
     #          ipnr.eq.ipnc.and.iqnr.eq.iqnc.and.
     #          ipsr.eq.ipsc.and.iqsr.eq.iqsc.and.
     #          imr.eq.imc.and.mr.eq.mc.and.
     #          abs(fact).gt.rndoff
c    #          ikr.eq.jk2*ikc
c    #          ikr.eq.jk2.and.ikc.eq.1
          if (check) then
             cgam=w3j(lr,2,lc,-kr,kk0a1,kc)+
     #            w3j(lr,2,lc,-kr,-kk0a1,kc)*ipkk01*jk2+
     #           (w3j(lr,2,lc,-kr,kk0a1,-kc)+
     #            w3j(lr,2,lc,-kr,-kk0a1,-kc)*ipkk01*jk2)*ikc*iskc
c    #            w3j(lr,2,lc,-kr,-kk0a1,-kc)*ipkk01*jk2)*ikc*iskc*jk2
             if (kki.eq.3) then
             cgam=cgam+
     #            w3j(lr,2,lc,-kr,kk0a2,kc)+
     #            w3j(lr,2,lc,-kr,-kk0a2,kc)*ipkk02*jk2+
     #           (w3j(lr,2,lc,-kr,kk0a2,-kc)+
     #            w3j(lr,2,lc,-kr,-kk0a2,-kc)*ipkk02*jk2)*ikc*iskc
c    #            w3j(lr,2,lc,-kr,-kk0a2,-kc)*ipkk02*jk2)*ikc*iskc*jk2
             endif
             cgam=cgam*fact
c
          endif
c
c----------------------------------------------------------------------
c             Store matrix element
c----------------------------------------------------------------------
c
           cliou=zero
c
c             Diagonal elements
c
              if (nrow.eq.ncol) then
                  zdiag(1,nrow)=cgam
                  zdiag(2,nrow)=zero
c                 write(*,*) 'zdizg',nrow,zdiag(1,nrow),zdiag(2,nrow)
                else
c
c              Off-diagonal elements
c
                  if (abs(cliou).gt.rndoff) then
                    nel=nel+1
                    if (nel.gt.mxel) then
                      ierr=1
                      go to 9999
                    else
                      neli=neli+1
                      zmat(neli)=-cliou
                      izmat(neli)=ncol
                    end if
                  end if
c
                  if (abs(cgam).gt.rndoff) then
                    nel=nel+1
                    if (nel.gt.mxel) then
                      ierr=1
                      go to 9999
                    else
                      nelr=nelr+1
         write(*,1000) nelr,lr,mr,kr,lc,mc,kc,cgam,nrow,ncol
 1000 format(2x,i5,',',2x,6(i2,','),f12.6,2(i2,','))
c        write(*,1002) nelr,jr,nr,ir,jc,nc,ic,cgam,nrow,ncol
c1002 format(2x,i5,',',2x,6(i2,','),f12.6,2(i2,','))
c        write(*,1001) ipnr,imr,iqnr,ipsr,iqsr,ipnc,imc,iqnc,ipsc,iqsc
c1001 format(2x,i5,',',2x,9(i2,','),i2)
                      zmat(mxel-nelr+1)=cgam
                      izmat(mxel-nelr+1)=ncol
                    end if
                  end if
                end if
c
c----------------------------------------------------------------------
c       end loop over columns
c----------------------------------------------------------------------
c
 300            continue
c
c----------------------------------------------------------------------
c     end loop over rows
c----------------------------------------------------------------------
c
 400  continue
c
c----------------------------------------------------------------------
c     return to calling program
c----------------------------------------------------------------------
c
      ierr=0
      neltot=nel
      nelre=nelr
      nelim=neli
c
      jzmat(ndim+1)=neli+1
      kzmat(ndim+1)=nelr+1
c
 9999 return
c
c9000 format (//,'Storing reduced matrix elements and other ',
c    #'parameters ... ',$)
c9010 format ('done.')
c
      end











