c
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c
c                               SRLS
c
c This code contains all subroutines and functions expecially written
c for the EPRSR calculation. They are listed in no particular order.
c
c List:
c
c     LSTORE [s] calculates reduced matrix elements for the
c                liouville operator.
c
c     ISYM [s]   picks up the right unsymmetrized elements.
c
c     CFPOT [s]  calculates and orders the F-expansion coefficients
c                in the diffusion operator.
c
c     CFAC [f]   calculates c+(l,m) or c-(l,m).
c
c     DELTA [f]  check the triangle condition.
c
c     GAMMA [s]  calculates the matrix element of the diffusion
c                operator in the unsymmetrized basis.
c
c     INTFC [s]  calculates the U-expansion coefficients
c                in the diffusion operator.
c
c     LIOUV [s]  calculates the matrix element of the liouville
c                operator in the unsymmetrized basis.
c
c     AMG [f]    matrix elements of A magnetic operators.
c
c     GMG [f]    matrix elements of g magnetic operators.
c
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c

c***********************************************************************
c
c     LSTORE
c
c                         Calculation of all reduced liouville matrix,
c                         i.e. the part of liouville m. e. not depending
c                         on L1 and L2. See Eqns. in the accompanying
c                         text.
c
c***********************************************************************
c
c     subroutine lstore
c
c     implicit none
c
c     include 'eprdat.inc'
c     include 'srsflg.inc'
c     include 'srsind.inc'
c     include 'srssto.inc'
c
c     integer ipnrmn,ipnrmx,iqnrmn,iqnrmx,
c    #        ipsrmn,ipsrmx,iqsrmn,iqsrmx
c     integer ipncmn,ipncmx,iqncmn,iqncmx,
c    #        ipscmn,ipscmx,iqscmn,iqscmx
c     integer mmm,kkk,m,mp,i,nrow,ncol
c     integer idx1,idx2,idx3,idx4,idx5,idx6,idx7,idx8,idx9,idx10
c
c     double precision c,d3,zero,a,d,amg,gmg
c
c     parameter (zero=0.d0)
c
c.......................................................................
c
c     d3=dsqrt(3.d0)
c     nrow=0
c
c----------------------------------------------------------------------
c     *** loop over ipn1 ***
c----------------------------------------------------------------------
c
c      ipnrmx=min(ipnmx,in2)
c      ipnrmn=-ipnrmx
c      do ipn1=ipnrmn,ipnrmx,1
c
c----------------------------------------------------------------------
c     *** loop over iqn1 ***
c----------------------------------------------------------------------
c
c      iqnrmx=in2-iabs(ipn1)
c      iqnrmn=-iqnrmx
c      do iqn1=iqnrmn,iqnrmx,2
c
c----------------------------------------------------------------------
c     *** loop over ips1 ***
c----------------------------------------------------------------------
c
c      ipsrmx=1
c      if (hfdflg) then
c        ipsrmn=-1
c      else
c        ipsrmn=1
c      end if
c
c      do ips1=ipsrmn,ipsrmx,1
c
c----------------------------------------------------------------------
c     *** loop over iqs1 ***
c----------------------------------------------------------------------
c
c      iqsrmx=1-iabs(ips1)
c      iqsrmn=-iqsrmx
c      do iqs1=iqsrmn,iqsrmx,2
c
c      nrow=nrow+1
c
c      ncol=0
c
c----------------------------------------------------------------------
c     *** loop over ipn2 ***
c----------------------------------------------------------------------
c
c        ipncmx=min(ipnmx,in2)
c        ipncmn=-ipncmx
c        do ipn2=ipncmn,ipncmx,1
c
c----------------------------------------------------------------------
c     *** loop over iqn2 ***
c----------------------------------------------------------------------
c
c        iqncmx=in2-iabs(ipn2)
c        iqncmn=-iqncmx
c        do iqn2=iqncmn,iqncmx,2
c
c----------------------------------------------------------------------
c     *** loop over ips2 ***
c----------------------------------------------------------------------
c
c        ipscmx=1
c        if (hfdflg) then
c          ipscmn=-1
c        else
c          ipscmn=1
c        end if
c
c        do ips2=ipscmn,ipscmx,1
c
c----------------------------------------------------------------------
c     *** loop over iqs2 ***
c----------------------------------------------------------------------
c
c        iqscmx=1-iabs(ips2)
c        iqscmn=-iqscmx
c        do iqs2=iqscmn,iqscmx,2
c
c        ncol=ncol+1
c
c        m=ipn1-ipn2+ips1-ips2
c
c........... rank zero ........................................
c
c        if (m.eq.0) then
c          c=-a0*amg(0,0)*d3
c
c..... in the high field approx. discard the constant term ....
c..... due to the isotropic electron Zeeman operator ..........
c
c          if (hfdflg) then
c            c=c-b0*gmg(0,0)*d3
c          end if
c        else
c          c=zero
c        end if
c
c        idx1=ipn1+in2+1
c        idx2=(iqn1+iqnrmx)/2+1
c        idx3=ips1+2
c        idx4=(iqs1+iqsrmx)/2+1
c        idx5=ipn2+in2+1
c        idx6=(iqn2+iqncmx)/2+1
c        idx7=ips2+2
c        idx8=(iqs2+iqscmx)/2+1
c
c        stl0(idx1,idx2,idx3,idx4,idx5,idx6,idx7,idx8)=c
c
c........... rank two .........................................
c
c..... loops on all possible M1-M2 and K1-K2 values ...........
c
c        i=0
c        do mmm=-2,2
c        do kkk=-2,2
c         i=i+1
c         if (iabs(m).le.2) then
c           c=zero
c           do mp=-2,2
c
c..... here enters the dipendence on the tilt angles ..........
c
c              d=d2psi(m+3,mmm+3)*d2bed(kkk+3,mp+3)
c              a=fgm(mp+3)*gmg(2,m)+fam(mp+3)*amg(2,m)
c              c=c+d*a
c           end do
c         else
c           c=zero
c         end if
c
c         idx9=mmm+3
c         idx10=kkk+3
c         stl2(idx9,idx10,idx1,idx2,idx3,idx4,idx5,idx6,idx7,idx8)=c
c
c        end do
c        end do
c
c        end do
c        end do
c        end do
c        end do
c
c      end do
c      end do
c      end do
c      end do
c
c......................................................................
c
c      return
c      end


c
c***********************************************************************
c
c     ISYM
c
c                         Symmetrization routines. Given the symmetrized
c                         quantum numbers, it defines one of the possible
c                         set of unsymmetrized quantum numbers, according
c                         to the values of i and j. See text.
c
c***********************************************************************
c
      subroutine isym(i,j)
c
c     implicit none
c
      include 'srsind.inc'
c
      integer i,j
c
c.......................................................................
c
      ll1=lr
      jj1=jr
      iqn1=iqnr
      ips1=ipsr
      iqs1=iqsr
c
      if (i.eq.1) then
         mm1=mr
         kk1=kr
         nn1=nr
         ii1=ir
         ipn1=ipnr
      else if (i.eq.2) then
         mm1=-mr
         kk1=kr
         nn1=-nr
         ii1=ir
         ipn1=-ipnr
      end if
c
      ll2=lc
      jj2=jc
      iqn2=iqnc
      ips2=ipsc
      iqs2=iqsc
c
      if (j.eq.1) then
         mm2=mc
         kk2=kc
         nn2=nc
         ii2=ic
         ipn2=ipnc
      else if (j.eq.2) then
         mm2=mc
         kk2=-kc
         nn2=nc
         ii2=ic
         ipn2=ipnc
      else if (j.eq.3) then
         mm2=mc
         kk2=kc
         nn2=nc
         ii2=-ic
         ipn2=ipnc
      else if (j.eq.4) then
         mm2=mc
         kk2=-kc
         nn2=nc
         ii2=-ic
         ipn2=ipnc
      else if (j.eq.5) then
         mm2=-mc
         kk2=kc
         nn2=-nc
         ii2=ic
         ipn2=-ipnc
      else if (j.eq.6) then
         mm2=-mc
         kk2=-kc
         nn2=-nc
         ii2=ic
         ipn2=-ipnc
      else if (j.eq.7) then
         mm2=-mc
         kk2=kc
         nn2=-nc
         ii2=-ic
         ipn2=-ipnc
      else if (j.eq.8) then
         mm2=-mc
         kk2=-kc
         nn2=-nc
         ii2=-ic
         ipn2=-ipnc
      end if
c         
      ! the convention I adopted when I wrote lstore and liouv,
      ! for reasons which I can not remember now (although I think
      ! that it was because, fundamentally, I am a masochist),
      ! involves using -ipn instead of ipn
 
      ipn1=-ipn1
      ipn2=-ipn2
c
      return
      end


c***********************************************************************
c
c     CFPOT
c
c                         F expansion in |lmk;jni> functions. See text.
c                         The f(lmk;jni) are stored as cf(ic) plus
c                         lf(ic), mf(ic), kf(ic), jf(ic), if(ic) 
c                         (ic=1,idfx),
c                         where idfx is the maximum number of exp.
c                         coefficients; ic=order number).
c                         Also, for each (m,k,i) it is
c                         defined idcf(m,k,i,id), with id going from
c                         1 to mxcf(m,k,i). See text.
c
c                         mxcf(m,k,i)   : max number of exp. coefficients
c                                         with values m,k,i; 
c
c                         idcf(m,k,i,id): order number corresponding
c                                         to the id-th coefficient
c                                         with values m,k,i.
c
c***********************************************************************
c
      subroutine cfpot
c
      include 'eprdat.inc'
      include 'pidef.inc'
      include 'rndoff.inc'
      include 'srsdat.inc'
c
c     implicit none
      integer ie,l,m,k,j,i,ia,l1,m1,k1,j1,i1,
     #        ib,l2,m2,k2,j2,i2,ipar,id,ma,ka
      double precision c,add,wg1,wg2,wg3,wg4,wg5,wg6,
     #                 fac1,fac2,pp1,pp2,dd1,dd2, 
     #                 fc0,fc1,fc2,fc3,fc4,fc5,
     #                 zero,pi8,w3j,cfac
      logical delta
c
      parameter (zero=0.d0,pi8=8.d0*pi*pi)
c
c------------------------------------------------------------------
c
c
c..... this is the literal translation of the expression for the ..
c..... i-th exp. coefficient given in the text ....................
c
      ie=0
      cmax=zero
      do l=0,mxpl
       do m=-l,l
        do k=-l,l
         do j=0,mxpl
          do i=-j,j
c
           c=zero
           do ia=1,idmx
            l1=ll(ia)
            m1=mm(ia)
            k1=kk(ia)
            j1=jj(ia)
            i1=ii(ia)
            if (l.eq.l1.and.m.eq.m1.and.k.eq.k1.and.
     #      j.eq.j1.and.i.eq.i1) then
             add=(drt*l*(l+1)+(dpt-drt)*k*k
     #       +d0rt*j*(j+1)+(d0pt-d0rt)*i*i)*cc(ia)/2.d0
             c=c+add
            end if
            do ib=1,idmx
             l2=ll(ib)
             m2=mm(ib)
             k2=kk(ib)
             j2=jj(ib)
             i2=ii(ib)
             if (m.eq.m1+m2.and.k.eq.k1+k2.and.
     #       i.eq.i1+i2.and.delta(l,l1,l2).and.delta(j,j1,j2)) then
c
              wg1=w3j(l,l1,l2,-m,m1,m2)
              wg2=w3j(l,l1,l2,-k,k1,k2)
              wg3=w3j(l,l1,l2,-k,k1-1,k2+1)
              wg4=w3j(j,j1,j2,-i,i1,i2)
              wg5=w3j(j,j1,j2,-i,i1-1,i2+1)
              wg6=w3j(j,j1,j2,m,-m1,-m2)
              fac1=cfac(-1,l1,k1)*cfac(1,l2,k2)
              fac2=cfac(-1,j1,i1)*cfac(1,j2,i2)
              pp1=dble(k1*k2)
              pp2=dble(i1*i2)
              dd1=dsqrt(dble((2*l+1)*(2*l1+1)*(2*l2+1)))
              dd2=dsqrt(dble((2*j+1)*(2*j1+1)*(2*j2+1)))
c
              fc0=-dble(ipar(k+i))/pi8/4.d0
              fc1=dd1*dd2
              fc2=wg1
              fc3=(dpt*pp1*wg2+drt*fac1*wg3)*wg4
     #            +(d0pt*pp2*wg4+d0rt*fac2*wg5)*wg2   
              fc4=wg6
              fc5=cc(ia)*cc(ib)
c
              add=fc0*fc1*fc2*fc3*fc4*fc5
c
              c=c+add
c
             end if
            end do
           end do
           if (dabs(c).gt.rndoff) then
             ie=ie+1
             lf(ie)=l
             mf(ie)=m
             kf(ie)=k
             jf(ie)=j
             if(ie)=i
             cf(ie)=c
             if (cmax.lt.dabs(c)) cmax=c
           end if
c
          end do
         end do
        end do
       end do
      end do
c
      idfx=ie
c
c..... discarding terms with a relative weight smaller than rndoff ...
c
      id=0
      do ie=1,idfx
         cf(ie)=cf(ie)/cmax
         if (dabs(cf(ie)).gt.rndoff) then
            id=id+1
            lf(id)=lf(ie)
            mf(id)=mf(ie)
            kf(id)=kf(ie)
            jf(id)=jf(ie)
            if(id)=if(ie)
            cf(id)=cf(ie)
c
c            write (*,9001) id,lf(ie),mf(ie),kf(ie),
c    #                         jf(ie),if(ie),cf(ie)*cmax
c9001       format(2x,6(i3,','),g13.6)
c
         end if
      end do
c
c      pause
c
      idfx=id
c     write (*,*) 'terms in f-expansion : ',idfx
c
c..... evaluating idcf and mxcf matrices .............................
c
      do m=-mxpl,mxpl
       do k=-mxpl,mxpl
        do i=-mxpl,mxpl
         id=0
         do ie=1,idfx
          ma=mf(ie)
          ka=kf(ie)
          ia=if(ie)
          if (m.eq.ma.and.k.eq.ka.and.i.eq.ia) then
            id=id+1
            idcf(m+mxpl+1,k+mxpl+1,i+mxpl+1,id)=ie
c           write(*,*) m,k,i,id,ie
          end if
         end do
         mxcf(m+mxpl+1,k+mxpl+1,i+mxpl+1)=id
        end do
       end do
      end do
c
      return
      end
c
c-----------------------------------------------------------------------
c
c     CFAC
c
c                    It evaluates c+(l,m) for is=1 and c-(l,m) for is=-1
c
c-----------------------------------------------------------------------
c

      function cfac(is,l,m)
c
c     implicit none
c
      integer l,m,is
      double precision cfac,zero
	parameter (zero=0.0D0)
c
      if ((is.ne.1.and.is.ne.-1).or.iabs(m).gt.l) then
        cfac=zero
      else
        cfac=dsqrt(dble(l*(l+1)-m*(m+is)))
      end if
c
      return
      end
c
c-----------------------------------------------------------------------
c
c     DELTA
c
c                    Check triangle condition among l1,l2,l3
c
c-----------------------------------------------------------------------
c
      function delta(lt1,lt2,lt3)
c
c     implicit none
c
      integer lt1,lt2,lt3
      logical delta
c
      if  ((iabs(lt2-lt3).le.lt1.and.lt1.le.lt2+lt3).and.
     #     (iabs(lt1-lt3).le.lt2.and.lt2.le.lt1+lt3).and.
     #     (iabs(lt1-lt2).le.lt3.and.lt3.le.lt1+lt2)) then
         delta=.true.
      else
         delta=.false.
      end if
c     
      return
      end
c
c***********************************************************************
c
c     GAMMA
c
c                         Gamma operator matrix element (unsymmetrized
c                         basis set)
c
c***********************************************************************
c
      subroutine gamma(c)
c
      include 'eprdat.inc'
      include 'pidef.inc'
      include 'rndoff.inc'
      include 'srsdat.inc'
      include 'srsind.inc'
c
c     implicit none
      integer idx,ipar,ma,ka,idal,mxal
      integer l,m,k,j,n,i
c
      double precision w3j,pi8,zero
      double precision c,add,c1
      logical check,delta
c
      parameter (zero=0.d0,pi8=8.d0*pi*pi)
c
c-------------------------------------------------------------------
c
      c=zero
c
c..... find m,k,n in the expansion which may give non-zero m. e. ...
c
      m=mm1-mm2
      k=kk1-kk2
      n=nn1-nn2
      i=ii1-ii2
c
c.... neglect cases in which m is not -n (property of F) ...........
c
      if (iabs(m).gt.mxpl.or.iabs(k).gt.mxpl.or.m+n.ne.0
     #                   .or.iabs(i).gt.mxpl) return
c
c.... retrieve f coefficients and calculates m. e. .................
c
      mxal=mxcf(m+mxpl+1,k+mxpl+1,i+mxpl+1)
      do idal=1,mxal
        idx=idcf(m+mxpl+1,k+mxpl+1,i+mxpl+1,idal)
        l=lf(idx)
        j=jf(idx)
        check=delta(ll1,l,ll2).and.delta(jj1,j,jj2).and.
     #  ipar(jj1+j+jj2).eq.1
        if (.not.check) then
         add=zero
        else
         add=w3j(ll1,l,ll2,mm1,-m,-mm2)*w3j(ll1,l,ll2,kk1,-k,-kk2)
     #   *w3j(jj1,j,jj2,nn1,-n,-nn2)*w3j(jj1,j,jj2,ii1,-i,-ii2)
     #   *dsqrt(dble((2*ll1+1)*(2*l+1)*(2*ll2+1)))
     #   *dsqrt(dble((2*jj1+1)*(2*j+1)*(2*jj2+1)))
     #   *ipar(mm1+kk1+nn1+ii1)*cf(idx)/pi8
        end if
        c=c+add
      end do
c
      c=c*cmax
c
c-------------------------------------------------------------------
c
      return
      end
c
c********************************************************************
c
c     INTFC
c
c                                 Stores coefficients of potential U
c                                 in ordered sets cc(i), ll(i), mm(i),
c                                 kk(i), jj(i), ii(i) (i=1,idmx; 
c                                 where idmx is the max number of terms.)
c
c********************************************************************
c
      subroutine intfc
c
      include 'eprdat.inc'
      include 'pidef.inc'
      include 'rndoff.inc'
      include 'srsdat.inc'
c
c     implicit none
c
      integer ia,l,m,k,j,i,ka,ipar
      double precision c,h2,h4,h6,g1,g2,f1,f2,f4,f6,zero,pi8,dlm
c     
      logical check1,check2,check4
c
      parameter (zero=0.d0,pi8=8*pi*pi)
c
c------------------------------------------------------------------
c
      h2=pi8/dsqrt(5.d0)
      h4=pi8/3.d0
      h6=pi8/dsqrt(13.d0)
      g1=pi8/dsqrt(3.d0)
      g2=pi8/dsqrt(5.d0)
      f2=pi8/5.d0
      f4=pi8/9.d0
      f6=pi8/13.d0
c
      ia=0
c
      do l=2,4,2
          do k=-l,l,2
            ka=iabs(k)
            c=zero
            if (l.eq.2.and.ka.eq.0) then
              c=h2*a20
            else if (l.eq.2.and.ka.eq.2) then
              c=h2*a22
            else if (l.eq.4.and.ka.eq.0) then
              c=h4*a40
            else if (l.eq.4.and.ka.eq.2) then
              c=h4*a42
            end if
            c=-c
            if (dabs(c).gt.rndoff) then
              ia=ia+1
              ll(ia)=l
              mm(ia)=0
              kk(ia)=k
              jj(ia)=0
              ii(ia)=0
              cc(ia)=c
            end if
          end do
      end do
c
      c=-g1*b10
      if (dabs(c).gt.rndoff) then
        ia=ia+1
        ll(ia)=0
        mm(ia)=0
        kk(ia)=0
        jj(ia)=1
        ii(ia)=0
        cc(ia)=c
      end if
c
      c=-g2*b20
      if (dabs(c).gt.rndoff) then
        ia=ia+1
        ll(ia)=0
        mm(ia)=0
        kk(ia)=0
        jj(ia)=2
        ii(ia)=0
        cc(ia)=c
      end if
c
      do l=2,4,2
        do m=-l,l       
          do k=-l,l,2
            do i=-l,l
            ka=iabs(k)
            c=zero
            if (l.eq.2.and.ka.eq.0) then
              c=f2*c20
            else if (l.eq.2.and.ka.eq.2) then
              c=f2*c22
            else if (l.eq.4.and.ka.eq.0) then
              c=f4*c40
            else if (l.eq.4.and.ka.eq.2) then
              c=f4*c42
            end if
            call dlm0(dlm,l,i,bag)
            c=-ipar(m)*c*dlm
            if (dabs(c).gt.rndoff) then
              ia=ia+1
              ll(ia)=l
              mm(ia)=m
              kk(ia)=k
              jj(ia)=l
              ii(ia)=i
              cc(ia)=c
            end if
            end do
          end do
        end do
      end do
c
      idmx=ia
c
c      write (*,'(//)')
c      do ia=1,idmx
c              write (*,9000) ia,ll(ia),mm(ia),kk(ia),
c    #                           jj(ia),ii(ia),cc(ia)
c9000         format(2x,6(i3,','),g13.6)
c      end do
c     write (*,*) 'terms in u-expansion : ',idmx
c      pause
c
c----------------------------------------------------------------------
c     determine mxpl: max l or j number in the F function of gamma op.
c----------------------------------------------------------------------
c
      check1=b10.ne.zero
      check2=a20.ne.zero.or.a22.ne.zero.or.b20.ne.zero.or.
     #       c20.ne.zero.or.c22.ne.zero
      check4=a40.ne.zero.or.a42.ne.zero.or.
     #       c40.ne.zero.or.c42.ne.zero
c
      mxpl=0
      if (check1) then
         mxpl=2
      end if
      if (check2) then
         mxpl=4
      end if
      if (check4) then
         mxpl=8
      end if
c     write(*,*) 'mxpl', mxpl

c
      return
      end

c
c##########################################################################
c
c     LIOUV
c
c                         Matrix element of the liouville operator in
c                         the unsymmetrized basis set.
c
c##########################################################################
c
c     subroutine liouv(cliou)
c
c     implicit none
c
c     include 'eprdat.inc'
c     include 'pidef.inc'
c     include 'rndoff.inc'
c     include 'srsdat.inc'
c     include 'srsind.inc'
c     include 'srsflg.inc'
c     include 'srssto.inc'
c
c     integer l,m,mp,mmm,kkk,i,i1,i2
c     integer ipar
c     integer idx1,idx2,idx3,idx4,idx5,idx6,idx7,idx8,idx9,idx10
c     integer iqnrmx,iqsrmx,iqncmx,iqscmx
c
c     double precision a,d,zero,fct0,fct1,d3
c     double precision w3j,gmg,amg,cliou
c
c     logical check,check0,check2,delta
c
c     parameter (zero=0.d0)
c
c--------------------------------------------------------------
c
c     d3=dsqrt(3.d0)
c
c     mmm=mm2-mm1
c     kkk=kk2-kk1
c     m=ipn1-ipn2+ips1-ips2
c
c     iqnrmx=in2-iabs(ipn1)
c     iqsrmx=1-iabs(ips1)
c     iqncmx=in2-iabs(ipn2)
c     iqscmx=1-iabs(ips2)
c     idx1=ipn1+in2+1
c     idx2=(iqn1+iqnrmx)/2+1
c     idx3=ips1+2
c     idx4=(iqs1+iqsrmx)/2+1
c     idx5=ipn2+in2+1
c     idx6=(iqn2+iqncmx)/2+1
c     idx7=ips2+2
c     idx8=(iqs2+iqscmx)/2+1
c     idx9=mmm+3
c     idx10=kkk+3
c
c--------------------------------------------------------------
c
c     check=jj1.eq.jj2.and.nn1.eq.nn2.and.ii1.eq.ii2
c
c     if (check) then
c
c........ liouville matrix element ............................
c
c       cliou=zero
c
c............. rank 0 .........................................
c
c       check0=ll1.eq.ll2.and.mmm.eq.0.and.kkk.eq.0.and.m.eq.0
c       if (check0) then
c         cliou=stl0(idx1,idx2,idx3,idx4,idx5,idx6,idx7,idx8)
c       end if
c
c............. rank 2 ..................................................
c
c       check2=delta(ll1,2,ll2).and.iabs(mmm).le.2.and.iabs(kkk).le.2
c    #  .and.iabs(m).le.2
c       if (check2) then
c         fct0=ipar(mm2-kk2)*dsqrt(dble((2*ll1+1)*(2*ll2+1)))
c         fct1=w3j(ll1,2,ll2,mm1,mmm,-mm2)
c    #    *w3j(ll1,2,ll2,kk1,kkk,-kk2)
c         cliou=cliou+fct0*fct1
c    #    *stl2(idx9,idx10,idx1,idx2,idx3,idx4,idx5,idx6,idx7,idx8)
c       end if
c
c     else
c       cliou=zero
c     end if
c
c-------------------------------------------------------------------------
c
c     return
c     end
c
c##########################################################################
c
c-------------------------------------------------------------------------
c
c     AMG
c
c                            Matrix elements of the A operators
c
c
c-------------------------------------------------------------------------
c
      function amg(l,m)
c
c-------------------------------------------------------------------------
c
c     implicit none
c
      include 'eprdat.inc'
      include 'srsind.inc'
c
      integer idps,idpn,idp,idqs,idqn,idq
      integer iaps,iapn,iaqs,iaqn,ipp,iqq
      integer ipar,l,m
      double precision a,sk,sa,w3j,d8,amg,ri
c
c.................................................................
c
      d8=dsqrt(8.d0)
      ri=in2/2.d0
c
      idps=ips1-ips2
      idpn=ipn1-ipn2
      idp=idps+idpn
      idqs=iqs1-iqs2
      idqn=iqn1-iqn2
      idq=idqs+idqn
c
      iaps=iabs(idps)
      iapn=iabs(idpn)
      iaqs=iabs(idqs)
      iaqn=iabs(idqn)
      ipp=idps*idpn
      iqq=idqs*idqn
c
       if (idp.eq.m.and.iaps.eq.iaqs.and.iapn.eq.iaqn.and.ipp.eq.iqq)
     # then
          a=ipar(idp)*dsqrt(2.d0*l+1.d0)*w3j(1,1,l,idps,idpn,-idp)
          sk=dsqrt(ri*(ri+1)-(iqn1*idqn+ipn1*idpn)
     #      *(iqn1*idqn+ipn1*idpn-2.d0)/4.d0)
          if (idps.eq.0.and.idpn.eq.0) then
            sa=(ips1*iqn1+ipn1*iqs1)/2.d0
          else if (idps.eq.0.and.idpn.ne.0) then
            sa=-(ips1*idpn+iqs1*idqn)*sk/d8
          else if (idps.ne.0.and.idpn.eq.0) then
            sa=-(ipn1*idps+iqn1*idqs)/d8
         else
            sa=idps*idqn*sk/2.d0
         end if
         a=a*sa
      else
         a=0.d0
      end if
c
      amg=a
c
      return
      end
c
c-------------------------------------------------------------------------
c
c     GMG
c
c                            Matrix elements of the g operators
c
c-------------------------------------------------------------------------
c
      function gmg(l,m)
c
c-------------------------------------------------------------------------
c
c     implicit none
c
      include 'srsind.inc'
c
      integer idps,idpn,idp,idqs,idqn,idq
      integer iaps,iapn,iaqs,iaqn,ipp,iqq,iaq,iap
      integer ipar,l,m
      double precision g,sg,w3j,d2,gmg
c
c...................................................................
c
      d2=dsqrt(2.d0)
c
      idps=ips1-ips2
      idpn=ipn1-ipn2
      idp=idps+idpn
      idqs=iqs1-iqs2
      idqn=iqn1-iqn2
      idq=idqs+idqn
c
      iaps=iabs(idps)
      iapn=iabs(idpn)
      iap=iabs(idpn)
      iaq=iabs(idqn)
c
       if (idpn.eq.0.and.idqn.eq.0.and.iaq.eq.iap.and.idp.eq.m)
     # then
          g=ipar(idp)*dsqrt(2.d0*l+1.d0)*w3j(1,1,l,idp,0,-idp)
          if (idps.eq.0) then
            sg=ips1+0.d0
         else
            sg=-idqs/d2
         end if
         g=g*sg
      else
         g=0.d0
      end if
c
      gmg=g
c
      return
      end
c
c**********************************************************************
c
c                       =========================
c                          subroutine DLM0
c                       =========================
c                                                             l
c       Calculate the reduced Wigner rotation matrix element d   (beta).
c                                                             m,0
c
c       written by ZL 28-Feb-95
c
c       Includes:
c               rndoff.inc
c               pidef.inc
c
c       Uses:
c
c**********************************************************************
c
c     subroutine dlm0(dlm,l,m,beta)
c
c     include 'pidef.inc'
c     include 'rndoff.inc'
c     include 'eprdat.inc'
c
c     integer l,m,mabs,mipar,ipar
c     double precision beta,dlm
c     double precision fct0,z,plgndr
c     external ipar,plgndr
c
c######################################################################
c
c     if(dabs(beta-90.0d0).lt.rndoff.or.
c    #   dabs(beta-270.0d0).lt.rndoff) then
c       z=0.0d0
c     else
c       z=dcos(beta*pi/180.0d0)
c     end if
c
c     if(m.lt.0) then
c       mipar=1
c     else
c       mipar=ipar(m)
c     end if
c
c     mabs=iabs(m)
c     fct0=fact(l-mabs)-fact(l+mabs)
c     fct0=dexp(fct0/2)*mipar
c
c     dlm=fct0*plgndr(l,mabs,z)
c
c     return
c     end
c
c       VERSION 1.0     12/17/88
c*********************************************************************
c
c                         FUNCTION FACT
c                         =============
c
c        This real function function returns the logarithm
c        of the factorial of the argument n (0 if n<=0)
c
c*********************************************************************
c
c     real function fact(n)
c
c     integer n,i
c
c     double precision zero
c     parameter (zero=0.d0)
c
c#####################################################################
c
c     if (n.le.1) then
c        fact=zero
c     else
c        fact=zero
c        do i=2,n
c           fact=fact+dlog(dble(i))
c        end do
c     end if
c
c     return
c     end
c
