c**********************************************************************
c                    =========================
c                       subroutine STVECT
c                    =========================
c                      *** Pruning version ***
c
c       Calculates the starting vector for EPRSR
c       
c       Notes:
c               1) The orientational integral is evaluated numerically 
c                  using the Curtis-Clenshaw-Romberg algorithm.
c                  For details, see Bruno's thesis, p. 554.
c
c       written by AP 17-FEB-94, based on STVECT for EPRLL written
c       by DJS 10-SEP-87 - Version 1.4, modified so that nelv is kept
c       in common eprdat instead of passed as a subroutine argument.
c       DEB 22-OCT-91
c
c       Includes:
c               stddim.inc
c               eprdat.inc
c               indexl.inc
c               rndoff.inc
c
c       Uses:
c               ccrint.f
c               fact.f  (included in the code)
c
c**********************************************************************
c
      subroutine stvect(v)
c
      implicit none
c
      include 'stddim.inc'
      include 'eprdat.inc'
      include 'indexl.inc'
      include 'rndoff.inc'
      include 'maxl.inc'
c
      double precision v(2,mxdim)
c
      integer nrow
      integer l,m,k,j,n,i,ipn,iqn,ips,iqs,im,ia,ik,ii,
     #                 ls,ms,ks,js,ns,is, 
     #                 lb,mb,kb,jb,nb,ib 
      integer jq,jbmx,neval,id,iff,jj,nn,ipar
c
      logical check,check1
c
      double precision stvt,vnorm,zero,fact,w3j,v0,dlm,stvt1
c
      double precision acc,sml,bndlow,bndhi,ans,fct0,fct1,fct2
      parameter (acc=1.0D-8,sml=1.0D-10,zero=0.d0)
c
      double precision vst(0:mxlval),
c    #     vtemp(0:mxlval1,-mxlval1:mxlval1,-mxlval1:mxlval1,
c    #           0:mxlval1,-mxlval1:mxlval1),
     #     vmean(0:mxlval1,-mxlval1:mxlval1)
c
      double precision fz
      external fz
c
      common /ifdat/ iff,jj,nn
c
c######################################################################
c
c
c......................................................................
c
      jbmx=lemx+jmx
      bndlow=-1.d0
      bndhi=1.d0
c
c.......... store cage integrals  .....................................
c
      iff=0
      do jq=0,jbmx
         jj=jq
         nn=0
         call ccrint(bndlow,bndhi,acc,sml,ans,neval,fz,id)
         vst(jq)=ans
      end do
c
      nrow=0
      nelv=0
      vnorm=0.0D0
c
c.......... store interaction integrals  ...............................
c
       bndlow=zero
       iff=1
c
c     do l=0,lemx
c     do m=-l,l
c     do k=0,l
c     do j=0,jmx
c     do i=-j,j
c
c     check=mod(l,2).eq.0.and.mod(k,2).eq.0.and.mod(j,2).eq.0
c    #      .and.iabs(m).le.j.and.iabs(i).le.l
c     if(check) then
c
c          jj=l
c          nn=k
c          call ccrint(bndlow,bndhi,acc,sml,ans,neval,fz,id)
c          fct0=fact(l-k)-fact(l+k)
c          fct0=dexp(fct0/2)*ans
c
c          if (k.ne.0) then
c             fct0=2*fct0
c          end if
c
cc          if (i.ne.0) then
cc             fct0=2*fct0
cc          end if
cc
c           call dlm0(dlm,l,i,bag)
cc          write(*,*) l,i,bag,dlm
cc          if(k.eq.0.and.m.eq.0.and.j.eq.0) write(*,*) l,i,bag,dlm
c         fct0=fct0*dsqrt(dble((2*l+1)*(2*j+1)))*ipar(i)*dlm
cc
c         v0=zero
c        do jq=iabs(l-j),l+j
c           fct1=(2*jq+1)*w3j(l,jq,j,-m,0,m)*w3j(l,jq,j,i,0,-i)
c           v0=v0+fct1*vst(jq)
c       end do
c
c          stvt=fct0*v0
c
c        else
c         stvt=zero
c      end if
c
c     vtemp(l,m,k,j,i)=stvt
c
c     end do
c    end do
c   end do
c  end do
c end do
c
c.......... store probe integrals  ...............................
c
      iff=2
c
      do l=0,lemx
      do k=0,l
c
         check=mod(l,2).eq.0.and.mod(k,2).eq.0
c
         if (check) then
c
           jj=l
           nn=k
           call ccrint(bndlow,bndhi,acc,sml,ans,neval,fz,id)
           fct0=fact(l-k)-fact(l+k)
           fct0=dexp(fct0/2)*ans
c
           if (k.ne.0) then
              fct0=2*fct0
           end if
c
           fct0=fct0*dsqrt(dble(2*l+1))
c
           stvt=fct0
c
         else
           stvt=zero
         end if
c
         if (abs(stvt).lt.rndoff) then
           vmean(l,k)=zero
         else
           vmean(l,k)=stvt
         end if
c
      end do
      end do
c
c----------------------------------------------------------------------
c     evaluate starting vector
c----------------------------------------------------------------------
c
      bndlow=zero
      iff=1
c
      do nrow=1,ndim
c
         l=l1(nrow)
         m=m1(nrow)
         k=k1(nrow)
         j=j1(nrow)
         n=n1(nrow)
         i=i1(nrow)
         ipn=pi1(nrow)
         im=im1(nrow)
         iqn=qi1(nrow)
         ips=ps1(nrow)
         iqs=qs1(nrow)
c
         check=ipn.eq.0.and.mod(l,2).eq.0.and.mod(k,2).eq.0.and.m.eq.-n
c
         if (check) then
c
            stvt=zero
c
            do lb=0,lemx
            do kb=0,lb
               mb=m
               jb=j
               nb=-mb
               ib=i
c
               check1=mod(lb,2).eq.0.and.mod(kb,2).eq.0.and.mod(jb,2)
     #                .eq.0.and.iabs(mb).le.jb.and.iabs(ib).le.lb
               if(check1) then
c
                 jj=lb
                 nn=kb
                 call ccrint(bndlow,bndhi,acc,sml,ans,neval,fz,id)
                 fct0=fact(lb-kb)-fact(lb+kb)
                 fct0=dexp(fct0/2)*ans
c
                 if (kb.ne.0) then
                    fct0=2*fct0
                 end if
c
                 call dlm0(dlm,lb,ib,bag)
                 fct0=fct0*dsqrt(dble((2*lb+1)*(2*jb+1)))*ipar(ib)*dlm
c
                 v0=zero
                 do jq=iabs(lb-jb),lb+jb
                    fct1=(2*jq+1)*w3j(lb,jq,jb,-mb,0,mb)
     #                           *w3j(lb,jq,jb,ib,0,-ib)
                    v0=v0+fct1*vst(jq)
                 end do
c
                 stvt1=fct0*v0
c
               else
                 stvt1=zero
               end if
c
               do ls=0,lemx
c              do ks=0,ls
                  ks=k-kb 
                  ms=0
                  js=0
                  ns=0
                  is=0
                  fct2=dsqrt(dble((2*l+1)*(2*ls+1)*(2*lb+1)))*
     #                 dsqrt(dble((2*j+1)*(2*js+1)*(2*jb+1)))*
     #                 ipar(m-k+n-i)*
     #                 w3j(l,ls,lb,m,-ms,-mb)*w3j(l,ls,lb,k,-ks,-kb)*
     #                 w3j(j,js,jb,n,-ns,-nb)*w3j(j,js,jb,i,-is,-ib)
                  stvt=stvt+fct2*vmean(ls,ks)*stvt1
               end do
c              end do
            end do
            end do
         else
            stvt=zero
         end if

         v(1,nrow)=stvt
         vnorm=vnorm+stvt*stvt
c
c        if(abs(stvt).gt.rndoff) write (*,1000) nrow,l,m,k,j,n,i,stvt
 1000 format(2x,i5,',',5(i2,','),3(f12.6))
      end do
c
c----------------------------------------------------------------------
c     normalize starting vector and zero out imaginary part
c----------------------------------------------------------------------
c
      nelv=0
      vnorm=1.0D0/dsqrt(vnorm)
      do 200 ia=1,ndim
        v(1,ia)=v(1,ia)*vnorm
        if(abs(v(1,ia)).gt.rndoff) then
          nelv=nelv+1
c         write(*,*) nelv,v(1,ia)
        else
          v(1,ia)=0.0D0
        end if
        v(2,ia)=0.0D0
 200  continue
c
c----------------------------------------------------------------------
c     zero out remainder of vector
c----------------------------------------------------------------------
c
      do 210 ia=ndim+1,mxdim
        v(1,ia)=0.0D0
        v(2,ia)=0.0D0
 210  continue        
c
      return
      end
c
c       VERSION 1.0     12/17/88
c*********************************************************************
c
c                         FUNCTION FACT
c                         =============
c
c        This real function function returns the logarithm
c        of the factorial of the argument n (0 if n<=0)
c
c*********************************************************************
c
c     double precision function fact(n)
c
c     integer n,i
c
c     double precision zero
c     parameter (zero=0.d0)
c
c#####################################################################
c
c     if (n.le.1) then
c        fact=zero
c     else
c        fact=zero
c        do i=2,n
c           fact=fact+dlog(dble(i))
c        end do
c     end if
c
c     return
c     end
c
c**********************************************************************
c
c                       =========================
c                          subroutine DLM0
c                       =========================
c                                                             l
c       Calculate the reduced Wigner rotation matrix element d   (beta).
c                                                             m,0
c
c       written by ZL 28-Feb-95
c
c       Includes:
c               rndoff.inc
c               pidef.inc
c
c       Uses:
c
c**********************************************************************
c
c     subroutine dlm0(dlm,l,m,beta)
c
c     include 'pidef.inc'
c     include 'rndoff.inc'
c     include 'eprdat.inc'
c
c     integer l,m,mabs,mipar,ipar
c     double precision beta,dlm
c     double precision fct0,fact,z,plgndr
c     external ipar,plgndr
c
c######################################################################
c
c     if(dabs(beta-90.0d0).lt.rndoff.or.
c    #   dabs(beta-270.0d0).lt.rndoff) then
c       z=0.0d0
c     else
c       z=dcos(beta*pi/180.0d0)
c     end if
c
c     if(m.lt.0) then
c       mipar=1
c     else
c       mipar=ipar(m)
c     end if
c
c     mabs=iabs(m)
c     fct0=fact(l-mabs)-fact(l+mabs)
c     fct0=dexp(fct0/2)*mipar
c
c     dlm=fct0*plgndr(l,mabs,z)
c
c     return
c     end
c
